@@ -4,7 +4,7 @@
 
   Qore Programming Language
 
-  Copyright (C) 2003 - 2016 David Nichols
+  Copyright (C) 2003 - 2018 Qore Technologies, s.r.o.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -188,10 +188,16 @@ AbstractQoreNode* OraColumnValue::getValue(ExceptionSink *xsink, bool horizontal
          return buf.lng->takeValue();
 
       case SQLT_RSET: {
-         QoreOracleStatement tstmt(stmt.getDatasource(), (OCIStmt*)buf.takePtr());
-         if (horizontal)
-            return tstmt.fetchRows(xsink);
-         return tstmt.fetchColumns(xsink);
+         if (!alt_output) {
+            QoreOracleStatement tstmt(stmt.getDatasource(), (OCIStmt*)buf.takePtr());
+            if (horizontal)
+               return tstmt.fetchRows(xsink);
+            return tstmt.fetchColumns(false, xsink);
+         }
+         else {
+            // return an SQLStatement object
+            return stmt.getDatasource()->getSQLStatementObjectForResultSet(new QorePreparedStatement(stmt.getDatasource(), (OCIStmt*)buf.takePtr()));
+         }
       }
 
       case SQLT_NTY: {
@@ -241,7 +247,6 @@ AbstractQoreNode* OraColumnValue::getValue(ExceptionSink *xsink, bool horizontal
       }
 #endif
 
-#ifdef _QORE_HAS_NUMBER_TYPE
       case SQLT_VNU:
       case SQLT_NUM: {
          int nopt = stmt.getData()->getNumberOption();
@@ -254,7 +259,6 @@ AbstractQoreNode* OraColumnValue::getValue(ExceptionSink *xsink, bool horizontal
          assert(nopt == OPT_NUM_NUMERIC);
          return new QoreNumberNode((const char*)buf.ptr);
       }
-#endif
    }
 
    // default: treat as string
