@@ -4,7 +4,7 @@
 
   Qore Programming Language
 
-  Copyright (C) 2003 - 2015 David Nichols
+  Copyright (C) 2003 - 2018 Qore Technologies, s.r.o.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -33,36 +33,39 @@ static int get_char_width(const QoreEncoding *enc, int num) {
 #endif
 }
 
-#define Q_LONG_BLOCK_SIZE LOB_BLOCK_SIZE
+#define Q_LONG_BLOCK_SIZE 4096
 static sb4 q_long_callback(q_lng* lng, OCIDefine *defnp, ub4 iter, void **bufpp, ub4 **alenpp, ub1 *piecep, void **indpp, ub2 **rcodep) {
-   //printd(5, "q_long_callback() lng: %p piece: %d alenp: %p (%d) indp: %p ind: %d size: %d str: %p\n", lng, *piecep, *alenpp, *alenpp ? **alenpp : 0, *indpp, (int)lng->ind, (int)lng->size, lng->str);
+   //printd(5, "q_long_callback() lng: %p iter: %d piece: %d alenp: %p (%d) indp: %p ind: %d size: %d str: %p\n", lng, iter, *piecep, *alenpp, *alenpp ? **alenpp : 0, *indpp, (int)lng->ind, (int)lng->size, lng->str);
 
    assert(!*indpp);
    *indpp = (void*)&lng->ind;
-   //lng->rc = 0;
-   //*indpp = 0;
+   *rcodep = &lng->rcode;
 
    switch (*piecep) {
       case OCI_FIRST_PIECE:
          if (!lng->str) {
             lng->str = new QoreStringNode(lng->enc);
-            lng->str->reserve(Q_LONG_BLOCK_SIZE);
+            lng->str->reserve(Q_LONG_BLOCK_SIZE + 1);
          }
-         lng->size = Q_LONG_BLOCK_SIZE;
          assert(lng->str->capacity());
          assert(lng->str->empty());
 
+         lng->size = Q_LONG_BLOCK_SIZE;
          *alenpp = &lng->size;
-         *bufpp = (void*)lng->str->getBuffer();
+         *bufpp = (void*)lng->str->c_str();
          break;
 
-      case OCI_NEXT_PIECE:
       case OCI_LAST_PIECE:
+      case OCI_NEXT_PIECE:
          assert(*alenpp);
+         assert(*alenpp == &lng->size);
          lng->str->terminate(lng->str->size() + lng->size);
-         //printd(5, "q_long_callback() str: %p size: %d capacity: %d '%s'\n", lng->str, lng->str->size(), lng->str->capacity(), lng->str->getBuffer());
-         lng->str->reserve(lng->str->size() + Q_LONG_BLOCK_SIZE);
-         *bufpp = (void*)(lng->str->getBuffer() + lng->str->size());
+         //printd(5, "q_long_callback() str: %p size: %d capacity: %d\n", lng->str, lng->str->size(), lng->str->capacity());
+         if (*piecep == OCI_NEXT_PIECE) {
+            lng->str->reserve(lng->str->size() + Q_LONG_BLOCK_SIZE + 1);
+            lng->size = Q_LONG_BLOCK_SIZE;
+            *bufpp = (void*)(lng->str->c_str() + lng->str->size());
+         }
          break;
    }
 
@@ -86,7 +89,7 @@ OraResultSet::OraResultSet(QoreOracleStatement &n_stmt, const char *str, Excepti
       text *col_name;
       ub4 col_name_len;
       ub2 col_max_size;
-      
+
       // get column type
       if (stmt.attrGet(parmp, &dtype, OCI_ATTR_DATA_TYPE, xsink))
          return;
@@ -158,103 +161,103 @@ int OraResultSet::define(const char *str, ExceptionSink *xsink) {
       OraColumnBuffer *w = clist[i];
       //printd(5, "OraResultSet::define() this=%p %s: w->dtype=%d\n", this, w->name.getBuffer(), w->dtype);
       switch (w->dtype) {
-	 case SQLT_INT:
-	 case SQLT_UIN:
-	    w->buf.i8 = 0;
+         case SQLT_INT:
+         case SQLT_UIN:
+            w->buf.i8 = 0;
             stmt.defineByPos(w->defp, i + 1, &w->buf.i8, sizeof(int64), SQLT_INT, &w->ind, xsink);
-	    break;
+            break;
 
-	 case SQLT_FLT:
+         case SQLT_FLT:
 #ifdef SQLT_BFLOAT
-	 case SQLT_BFLOAT:
+         case SQLT_BFLOAT:
 #endif
 #ifdef SQLT_BDOUBLE
-	 case SQLT_BDOUBLE:
+         case SQLT_BDOUBLE:
 #endif
 #ifdef SQLT_IBFLOAT
-	 case SQLT_IBFLOAT:
+         case SQLT_IBFLOAT:
 #endif
 #ifdef SQLT_IBDOUBLE
-	 case SQLT_IBDOUBLE:
+         case SQLT_IBDOUBLE:
 #endif
 #if defined(SQLT_BDOUBLE) && defined(USE_NEW_NUMERIC_TYPES)
-	    stmt.defineByPos(w->defp, i + 1, &w->buf.f8, sizeof(double), SQLT_BDOUBLE, &w->ind, xsink);
+            stmt.defineByPos(w->defp, i + 1, &w->buf.f8, sizeof(double), SQLT_BDOUBLE, &w->ind, xsink);
 #else
-	    stmt.defineByPos(w->defp, i + 1, &w->buf.f8, sizeof(double), SQLT_FLT, &w->ind, xsink);
+            stmt.defineByPos(w->defp, i + 1, &w->buf.f8, sizeof(double), SQLT_FLT, &w->ind, xsink);
 #endif
-	    break;
+            break;
 
-	 case SQLT_DAT:
-	    stmt.defineByPos(w->defp, i + 1, w->buf.date, 7, SQLT_DAT, &w->ind, xsink);
-	    break;
+         case SQLT_DAT:
+            stmt.defineByPos(w->defp, i + 1, w->buf.date, 7, SQLT_DAT, &w->ind, xsink);
+            break;
 
-	 case SQLT_TIMESTAMP:
-	 case SQLT_TIMESTAMP_TZ:
-	 case SQLT_TIMESTAMP_LTZ:
-	 case SQLT_DATE:
-	    w->buf.odt = NULL;
+         case SQLT_TIMESTAMP:
+         case SQLT_TIMESTAMP_TZ:
+         case SQLT_TIMESTAMP_LTZ:
+         case SQLT_DATE:
+            w->buf.odt = NULL;
             if (conn->descriptorAlloc((dvoid **)&w->buf.odt, QORE_DTYPE_TIMESTAMP, str, xsink))
                return -1;
-	    //printd(5, "OraResultSet::define() this=%p got TIMESTAMP handle: %p size: %d\n", this, w->buf.odt, sizeof(w->buf.odt));
-	    stmt.defineByPos(w->defp, i + 1, &w->buf.odt, sizeof(w->buf.odt), QORE_SQLT_TIMESTAMP, &w->ind, xsink);
-	    break;
+            //printd(5, "OraResultSet::define() this=%p got TIMESTAMP handle: %p size: %d\n", this, w->buf.odt, sizeof(w->buf.odt));
+            stmt.defineByPos(w->defp, i + 1, &w->buf.odt, sizeof(w->buf.odt), QORE_SQLT_TIMESTAMP, &w->ind, xsink);
+            break;
 
-	 case SQLT_INTERVAL_YM:
-	    w->buf.oi = NULL;
+         case SQLT_INTERVAL_YM:
+            w->buf.oi = NULL;
             if (conn->descriptorAlloc((dvoid **)&w->buf.oi, OCI_DTYPE_INTERVAL_YM, str, xsink))
                return -1;
-	    //printd(5, "OraResultSet::define() got INTERVAL_YM handle %p\n", w->buf.oi);
-	    stmt.defineByPos(w->defp, i + 1, &w->buf.oi, sizeof(w->buf.oi), SQLT_INTERVAL_YM, &w->ind, xsink);
-	    break;
+            //printd(5, "OraResultSet::define() got INTERVAL_YM handle %p\n", w->buf.oi);
+            stmt.defineByPos(w->defp, i + 1, &w->buf.oi, sizeof(w->buf.oi), SQLT_INTERVAL_YM, &w->ind, xsink);
+            break;
 
-	 case SQLT_INTERVAL_DS:
-	    w->buf.oi = NULL;
+         case SQLT_INTERVAL_DS:
+            w->buf.oi = NULL;
             if (conn->descriptorAlloc((dvoid **)&w->buf.oi, OCI_DTYPE_INTERVAL_DS, str, xsink))
                return -1;
-	    //printd(5, "OraResultSet::define() got INTERVAL_DS handle %p\n", w->buf.oi);
-	    stmt.defineByPos(w->defp, i + 1, &w->buf.oi, sizeof(w->buf.oi), SQLT_INTERVAL_DS, &w->ind, xsink);
-	    break;
+            //printd(5, "OraResultSet::define() got INTERVAL_DS handle %p\n", w->buf.oi);
+            stmt.defineByPos(w->defp, i + 1, &w->buf.oi, sizeof(w->buf.oi), SQLT_INTERVAL_DS, &w->ind, xsink);
+            break;
 
-	 // handle raw data
-	 case SQLT_BIN:
-	 case SQLT_LBI: {
-	    int size = w->maxsize;
-	    if (!size)
-	       size = ORA_RAW_SIZE;
+         // handle raw data
+         case SQLT_BIN:
+         case SQLT_LBI: {
+            int size = w->maxsize;
+            if (!size)
+               size = ORA_RAW_SIZE;
 
-	    w->buf.ptr = 0;
+            w->buf.ptr = 0;
             if (conn->rawResize((OCIRaw**)&w->buf.ptr, size, xsink))
-	       return -1;
+               return -1;
 
-	    stmt.defineByPos(w->defp, i + 1, w->buf.ptr, size + sizeof(int), SQLT_LVB, &w->ind, xsink);
-	    //printd(5, "OraResultSet::define() w=%p SQLT_LVB size=%d ptr=%p\n", w, size, w->buf.ptr);
-	    break;
-	 }
+            stmt.defineByPos(w->defp, i + 1, w->buf.ptr, size + sizeof(int), SQLT_LVB, &w->ind, xsink);
+            //printd(5, "OraResultSet::define() w=%p SQLT_LVB size=%d ptr=%p\n", w, size, w->buf.ptr);
+            break;
+         }
 
-	 case SQLT_BLOB:
-	 case SQLT_CLOB:
-	    w->buf.ptr = 0;
+         case SQLT_BLOB:
+         case SQLT_CLOB:
+            w->buf.ptr = 0;
             if (conn->descriptorAlloc(&w->buf.ptr, OCI_DTYPE_LOB, str, xsink))
                return -1;
-	    //printd(5, "OraResultSet::define() got LOB locator handle %p\n", w->buf.ptr);
-	    stmt.defineByPos(w->defp, i + 1, &w->buf.ptr, 0, w->dtype, &w->ind, xsink);
-	    break;
+            //printd(5, "OraResultSet::define() got LOB locator handle %p\n", w->buf.ptr);
+            stmt.defineByPos(w->defp, i + 1, &w->buf.ptr, 0, w->dtype, &w->ind, xsink);
+            break;
 
          case SQLT_LNG:
-	    w->buf.lng = new q_lng(stmt.getEncoding());
-            if (stmt.defineByPos(w->defp, i + 1, 0, LOB_BLOCK_SIZE, SQLT_CHR, &w->ind, xsink, OCI_DYNAMIC_FETCH))
+            w->buf.lng = new q_lng(stmt.getEncoding());
+            if (stmt.defineByPos(w->defp, i + 1, 0, 1024 * 1024 * 10, SQLT_CHR, &w->ind, xsink, OCI_DYNAMIC_FETCH))
                return -1;
 
             stmt.defineDynamic(w->defp, w->buf.lng, (OCICallbackDefine)q_long_callback, xsink);
             break;
 
-	 case SQLT_RSET:
-	    w->buf.ptr = 0;
-	    // allocate statement handle for result list
-	    if (conn->handleAlloc(&w->buf.ptr, OCI_HTYPE_STMT, str, xsink))
+         case SQLT_RSET:
+            w->buf.ptr = 0;
+            // allocate statement handle for result list
+            if (conn->handleAlloc(&w->buf.ptr, OCI_HTYPE_STMT, str, xsink))
                return -1;
-	    stmt.defineByPos(w->defp, i + 1, &w->buf.ptr, 0, w->dtype, &w->ind, xsink);
-	    break;
+            stmt.defineByPos(w->defp, i + 1, &w->buf.ptr, 0, w->dtype, &w->ind, xsink);
+            break;
 
          case SQLT_NTY: {
             // w->ind is not affected in the OCIDefineByPos for SQLT_NTY. But set it to 0.
@@ -262,8 +265,8 @@ int OraResultSet::define(const char *str, ExceptionSink *xsink) {
             w->ind = 0;
             if (w->subdtype == SQLT_NTY_OBJECT) {
                 w->buf.oraObj = objPlaceholderQore(conn, w->subdtypename.getBuffer(), xsink);
-		if (*xsink)
-		   return -1;
+                if (*xsink)
+                   return -1;
 
                 stmt.defineByPos(w->defp, i + 1, 0, 0, w->dtype, &w->ind, xsink);
                 conn->checkerr(OCIDefineObject((OCIDefine *) w->defp,
@@ -291,29 +294,29 @@ int OraResultSet::define(const char *str, ExceptionSink *xsink) {
             }
             break;
          } // SQLT_NTY
-            
+
 #ifdef SQLT_RDD
          case SQLT_RDD:
-	    w->buf.ptr = 0;
+            w->buf.ptr = 0;
             if (conn->descriptorAlloc(&w->buf.ptr, OCI_DTYPE_ROWID, str, xsink))
                return -1;
-	    //printd(5, "OraResultSet::define() got ROWID descriptor %p\n", w->buf.ptr);
-	    stmt.defineByPos(w->defp, i + 1, &w->buf.ptr, 0, w->dtype, &w->ind, xsink);
-	    break;
+            //printd(5, "OraResultSet::define() got ROWID descriptor %p\n", w->buf.ptr);
+            stmt.defineByPos(w->defp, i + 1, &w->buf.ptr, 0, w->dtype, &w->ind, xsink);
+            break;
 #endif
 
          case SQLT_NUM:
             w->maxsize = ORACLE_NUMBER_STR_LEN;
-	    w->buf.ptr = malloc(sizeof(char) * (w->maxsize + 1));
-	    stmt.defineByPos(w->defp, i + 1, w->buf.ptr, w->maxsize + 1, SQLT_STR, &w->ind, xsink);
+            w->buf.ptr = malloc(sizeof(char) * (w->maxsize + 1));
+            stmt.defineByPos(w->defp, i + 1, w->buf.ptr, w->maxsize + 1, SQLT_STR, &w->ind, xsink);
             break;
 
          default: // treated as a string
-	    if (w->charlen)
-               w->maxsize = get_char_width(stmt.getEncoding(), w->charlen); 
-	    w->buf.ptr = malloc(sizeof(char) * (w->maxsize + 1));
+            if (w->charlen)
+               w->maxsize = get_char_width(stmt.getEncoding(), w->charlen);
+            w->buf.ptr = malloc(sizeof(char) * (w->maxsize + 1));
             //printd(0, "OraResultSet::define() i=%d, buf=%p, maxsize=%d\n", i + 1, w->buf.ptr, w->maxsize);
-	    stmt.defineByPos(w->defp, i + 1, w->buf.ptr, w->maxsize + 1, SQLT_STR, &w->ind, xsink);
+            stmt.defineByPos(w->defp, i + 1, w->buf.ptr, w->maxsize + 1, SQLT_STR, &w->ind, xsink);
             break;
       }
       if (*xsink) return -1;
@@ -321,4 +324,3 @@ int OraResultSet::define(const char *str, ExceptionSink *xsink) {
 
    return 0;
 }
-
