@@ -4,7 +4,7 @@
 
   Qore Programming Language
 
-  Copyright (C) 2003 - 2016 David Nichols
+  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -22,14 +22,7 @@
 */
 
 #include "oracle.h"
-
-static inline bool wasInTransaction(Datasource* ds) {
-#ifdef _QORE_HAS_DATASOURCE_ACTIVETRANSACTION
-   return ds->activeTransaction();
-#else
-   return ds->isInTransaction();
-#endif
-}
+#include "ocilib/ocilib_internal.h"
 
 int QoreOracleStatement::setupDateDescriptor(OCIDateTime*& odt, ExceptionSink* xsink) {
    if (conn.descriptorAlloc((dvoid**)&odt, QORE_DTYPE_TIMESTAMP, "QoreOracleStatement::setupDateDecriptor()", xsink))
@@ -66,77 +59,6 @@ int QoreOracleSimpleStatement::exec(const char* sql, unsigned len, ExceptionSink
    return conn.checkerr(OCIStmtExecute(conn.svchp, stmthp, conn.errhp, 1, 0, 0, 0, OCI_DEFAULT), "QoreOracleSimpleStatement::exec", xsink);
 }
 
-int QoreOracleStatement::execute(ExceptionSink* xsink, const char* who) {
-   assert(conn.svchp);
-   ub4 iters;
-   if (is_select)
-      iters = 0;
-   else
-      iters = !array_size ? 1 : array_size;
-   int status = OCIStmtExecute(conn.svchp, stmthp, conn.errhp, iters, 0, 0, 0, OCI_DEFAULT);
-
-   //printd(0, "QoreOracleStatement::execute() stmthp: %p status: %d (OCI_ERROR: %d)\n", stmthp, status, OCI_ERROR);
-   if (status == OCI_ERROR) {
-      // see if server is connected
-      int ping = OCI_Ping(&conn.ocilib, conn.ocilib_cn, xsink);
-
-      if (!ping) {
-	 // check if a transaction was in progress
-         if (wasInTransaction(ds))
-	    xsink->raiseException("DBI:ORACLE:TRANSACTION-ERROR", "connection to Oracle database server %s@%s lost while in a transaction; transaction has been lost", ds->getUsername(), ds->getDBName());
-
-	 // try to reconnect
-	 conn.logoff();
-
-	 //printd(0, "QoreOracleStatement::execute() about to execute OCILogon() for reconnect\n");
-	 if (conn.logon(xsink)) {
-            //printd(5, "QoreOracleStatement::execute() conn: %p reconnect failed, marking connection as closed\n", conn);
-            // free current statement state while the driver-specific context data is still present
-            clearAbortedConnection(xsink);
-	    // close datasource and remove private data
-	    ds->connectionAborted();
-	    return -1;
-	 }
-         if (conn.checkWarnings(xsink)) {
-             //printd(5, "QoreOracleStatement::execute() conn: %p reconnect failed, marking connection as closed\n", conn);
-            // free current statement state while the driver-specific context data is still present
-            clearAbortedConnection(xsink);
-            // close datasource and remove private data
-            ds->connectionAborted();
-            return -1;
-         }
-
-         // don't execute again if the connection was aborted while in a transaction
-         if (wasInTransaction(ds))
-	    return -1;
-
-         if (resetAbortedConnection(xsink))
-            return -1;
-
-#ifdef DEBUG
-         // otherwise show the exception on stdout in debug mode
-         xsink->handleExceptions();
-#endif
-         // clear any exceptions that have been ignored
-         xsink->clear();
-
-	 //printd(0, "QoreOracleStatement::execute() returned from OCILogon() status: %d\n", status);
-	 status = OCIStmtExecute(conn.svchp, stmthp, conn.errhp, iters, 0, 0, 0, OCI_DEFAULT);
-	 if (status && conn.checkerr(status, who, xsink))
-	    return -1;
-      }
-      else {
-	 //printd(0, "QoreOracleStatement::execute() error, but it's connected; status: %d who: %s\n", status, who);
-	 conn.checkerr(status, who, xsink);
-	 return -1;
-      }
-   }
-   else if (status && conn.checkerr(status, who, xsink))
-      return -1;
-
-   return 0;
-}
-
 QoreHashNode* QoreOracleStatement::fetchRow(OraResultSet& resultset, ExceptionSink* xsink) {
    if (!fetch_done) {
       xsink->raiseException("ORACLE-FETCH-ROW-ERROR", "call SQLStatement::next() before calling SQLStatement::fetchRow()");
@@ -173,7 +95,7 @@ QoreHashNode* QoreOracleStatement::fetchRow(OraResultSet& resultset, ExceptionSi
 
       hah.assign(n, xsink);
       if (*xsink)
-	 return 0;
+         return 0;
    }
 
    return h.release();
@@ -211,13 +133,13 @@ QoreListNode* QoreOracleStatement::fetchRows(OraResultSet& resultset, int rows,
    while (next(xsink)) {
       QoreHashNode* h = fetchRow(resultset, xsink);
       if (!h)
-	 return 0;
+         return 0;
 
       // add row to list
       l->push(h);
 
       if (rows > 0 && l->size() == static_cast<size_t>(rows))
-	 break;
+         break;
    }
    //printd(2, "QoreOracleStatement::fetchRows(): %d column(s), %d row(s) retrieved as output\n", resultset.size(), l->size());
    if (!*xsink) {
@@ -230,7 +152,6 @@ QoreListNode* QoreOracleStatement::fetchRows(OraResultSet& resultset, int rows,
    return 0;
 }
 
-#ifdef _QORE_HAS_DBI_SELECT_ROW
 QoreHashNode* QoreOracleStatement::fetchSingleRow(ExceptionSink* xsink) {
    OraResultSetHelper resultset(*this, "QoreOracleStatement::fetchRow():params", xsink);
    if (*xsink)
@@ -259,25 +180,32 @@ QoreHashNode* QoreOracleStatement::fetchSingleRow(ExceptionSink* xsink) {
       fetch_done = true;
 
    if (next(xsink)) {
-      xsink->raiseExceptionArg("ORACLE-SELECT-ROW-ERROR", rv.release(), "SQL passed to selectRow() returned more than 1 row");
+      xsink->raiseExceptionArg("DBI-SELECT-ROW-ERROR", rv.release(), "SQL passed to selectRow() returned more than 1 row");
       return 0;
    }
 
    return rv.release();
 }
-#endif
+
+void QoreOracleStatement::doColumns(OraResultSet& resultset, QoreHashNode& h) {
+   // create hash elements for each column, assign empty list
+   for (clist_t::iterator i = resultset.clist.begin(), e = resultset.clist.end(); i != e; ++i) {
+      //printd(5, "QoreOracleStatement::fetchColumns() allocating list for '%s' column\n", w->name);
+      h.setKeyValue((*i)->name, new QoreListNode, 0);
+   }
+}
 
 // retrieve results from statement and return hash
-QoreHashNode* QoreOracleStatement::fetchColumns(ExceptionSink* xsink) {
+QoreHashNode* QoreOracleStatement::fetchColumns(bool cols, ExceptionSink* xsink) {
    OraResultSetHelper resultset(*this, "QoreOracleStatement::fetchColumns():params", xsink);
    if (*xsink)
       return 0;
 
-   return fetchColumns(**resultset, -1, xsink);
+   return fetchColumns(**resultset, -1, cols, xsink);
 }
 
 // retrieve results from statement and return hash
-QoreHashNode* QoreOracleStatement::fetchColumns(OraResultSet& resultset, int rows, ExceptionSink* xsink) {
+QoreHashNode* QoreOracleStatement::fetchColumns(OraResultSet& resultset, int rows, bool cols, ExceptionSink* xsink) {
    if (fetch_warned) {
       xsink->raiseException("ORACLE-SELECT-COLUMNS-ERROR", "SQLStatement::fetchColumns() called after the end of data already received");
       return 0;
@@ -287,6 +215,7 @@ QoreHashNode* QoreOracleStatement::fetchColumns(OraResultSet& resultset, int row
    ReferenceHolder<QoreHashNode> h(new QoreHashNode, xsink);
 
    if (fetch_complete) {
+      assert(!cols);
       fetch_warned = true;
       return h.release();
    }
@@ -294,12 +223,6 @@ QoreHashNode* QoreOracleStatement::fetchColumns(OraResultSet& resultset, int row
    if (setPrefetch(xsink, rows))
       return 0;
 
-   // create hash elements for each column, assign empty list
-   for (clist_t::iterator i = resultset.clist.begin(), e = resultset.clist.end(); i != e; ++i) {
-      //printd(5, "QoreOracleStatement::fetchColumns() allocating list for '%s' column\n", w->name);
-      h->setKeyValue((*i)->name, new QoreListNode, xsink);
-   }
-
    // setup temporary row to accept values
    if (resultset.define("QoreOracleStatement::fetchColumns():define", xsink))
       return 0;
@@ -309,15 +232,21 @@ QoreHashNode* QoreOracleStatement::fetchColumns(OraResultSet& resultset, int row
    // tracks the column size for when columns are duplicated
    unsigned csize = 0;
 
+   if (cols)
+      doColumns(resultset, **h);
+
    // now finally fetch the data
    while (next(xsink)) {
+      if (h->empty())
+         doColumns(resultset, **h);
+
       // copy data or perform per-value processing if needed
       for (unsigned i = 0; i < resultset.clist.size(); ++i) {
-	 OraColumnBuffer *w = resultset.clist[i];
-	 // get pointer to value of target node
-	 QoreListNode* l = reinterpret_cast<QoreListNode*>(h->getKeyValue(w->name, xsink));
-	 if (!l)
-	    break;
+         OraColumnBuffer *w = resultset.clist[i];
+         // get pointer to value of target node
+         QoreListNode* l = reinterpret_cast<QoreListNode*>(h->getKeyValue(w->name, xsink));
+         if (!l)
+            break;
 
          if (!i)
             csize = l->size();
@@ -349,14 +278,14 @@ QoreHashNode* QoreOracleStatement::fetchColumns(OraResultSet& resultset, int row
             break;
          }
 
-	 l->push(n);
-	 if (*xsink)
-	    break;
+         l->push(n);
+         if (*xsink)
+            break;
       }
 
       ++num_rows;
       if (rows > 0 && num_rows == rows)
-	 break;
+         break;
    }
    //printd(2, "QoreOracleStatement::fetchColumns(rows: %d): %d column(s), %d row(s) retrieved as output\n", rows, resultset.size(), num_rows);
    if (!*xsink) {
@@ -369,7 +298,6 @@ QoreHashNode* QoreOracleStatement::fetchColumns(OraResultSet& resultset, int row
    return 0;
 }
 
-#ifdef _QORE_HAS_DBI_DESCRIBE
 QoreHashNode* QoreOracleStatement::describe(OraResultSet& resultset, ExceptionSink* xsink) {
    // set up hash for row
    ReferenceHolder<QoreHashNode> h(new QoreHashNode, xsink);
@@ -479,4 +407,3 @@ QoreHashNode* QoreOracleStatement::describe(OraResultSet& resultset, ExceptionSi
 
    return h.release();
 }
-#endif
