@@ -4,7 +4,7 @@
 
   Qore Programming Language
 
-  Copyright (C) 2003 - 2016 David Nichols
+  Copyright (C) 2003 - 2018 Qore Technologies, s.r.o.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -44,7 +44,7 @@ struct QoreOracleSimpleStatement {
    QoreOracleConnection& conn;
    OCIStmt* stmthp;
 
-   DLLLOCAL QoreOracleSimpleStatement(QoreOracleConnection& n_conn, OCIStmt* n_stmthp = 0) : conn(n_conn), stmthp(n_stmthp) {
+   DLLLOCAL QoreOracleSimpleStatement(QoreOracleConnection& n_conn, OCIStmt* n_stmthp = nullptr) : conn(n_conn), stmthp(n_stmthp) {
    }
 
    DLLLOCAL ~QoreOracleSimpleStatement() {
@@ -75,7 +75,7 @@ struct QoreOracleSimpleStatement {
 
    DLLLOCAL int prepare(const char* sql, unsigned len, ExceptionSink* xsink) {
       return conn.checkerr(OCIStmtPrepare(stmthp, conn.errhp, (text*)sql, len, OCI_NTV_SYNTAX, OCI_DEFAULT),
-			     "QoreOracleSimpleStatement::prepare()", xsink);
+                             "QoreOracleSimpleStatement::prepare()", xsink);
    }
 
    DLLLOCAL int exec(const char* sql, unsigned len, ExceptionSink* xsink);
@@ -99,17 +99,7 @@ public:
    DLLLOCAL QoreOracleStatement(Datasource* n_ds, OCIStmt* n_stmthp = 0) : QoreOracleSimpleStatement(n_ds->getPrivateDataRef<QoreOracleConnection>(), n_stmthp), ds(n_ds), array_size(0), prefetch_rows(1), is_select(false), fetch_done(false), fetch_complete(false), fetch_warned(false) {
    }
 
-   DLLLOCAL virtual ~QoreOracleStatement() {
-   }
-
-   // this virtual function is called when the connection is closed while executing SQL so that
-   // the current state can be freed while the driver-specific context data is still present
-   DLLLOCAL virtual void clearAbortedConnection(ExceptionSink* xsink) {
-   }
-
-   // this virtual function is called after the connection has been closed and reopened while executing SQL
-   DLLLOCAL virtual int resetAbortedConnection(ExceptionSink* xsink) {
-      return 0;
+   DLLLOCAL ~QoreOracleStatement() {
    }
 
    DLLLOCAL void reset(ExceptionSink* xsink) {
@@ -149,11 +139,11 @@ public:
       int status;
 
       if ((status = OCIStmtFetch2(stmthp, conn.errhp, rows, OCI_FETCH_NEXT, 0, OCI_DEFAULT))) {
-	 if (status == OCI_NO_DATA) {
+         if (status == OCI_NO_DATA) {
             //printd(5, "QoreOracleStatement::fetch() this=%p no more data\n", this);
-	    return 1;
+            return 1;
          }
-	 else if (conn.checkerr(status, "QoreOracleStatement::fetch()", xsink)) {
+         else if (conn.checkerr(status, "QoreOracleStatement::fetch()", xsink)) {
             //printd(5, "QoreOracleStatement::fetch() this=%p exception\n", this);
             return -1;
          }
@@ -170,31 +160,29 @@ public:
 
    DLLLOCAL int defineByPos(OCIDefine*& defp, unsigned pos, void* valuep, int value_sz, unsigned short dty, void* indp, ExceptionSink* xsink, ub4 mode = OCI_DEFAULT) {
       return conn.checkerr(OCIDefineByPos(stmthp, &defp, conn.errhp, pos, valuep, value_sz, dty, indp, 0, 0, mode),
-			     "QoreOracleStatement::defineByPos()", xsink);
+                             "QoreOracleStatement::defineByPos()", xsink);
    }
 
    DLLLOCAL int bindByPos(OCIBind*& bndp, unsigned pos, void* valuep, int value_sz, unsigned short dty, ExceptionSink* xsink, void* indp, ub4 mode = OCI_DEFAULT) {
       return conn.checkerr(OCIBindByPos(stmthp, &bndp, conn.errhp, pos, valuep, value_sz, dty, indp, 0, 0, 0, 0, mode),
-			     "QoreOracleStatement::bindByPos()", xsink);
+                             "QoreOracleStatement::bindByPos()", xsink);
    }
 
    DLLLOCAL int prepare(QoreString& str, ExceptionSink* xsink) {
       int rc = QoreOracleSimpleStatement::prepare(str.getBuffer(), str.strlen(), xsink);
       if (!rc) {
-	 // see what kind of statement was prepared and set the flag accordingly
-	 ub2 stype;
-	 if (attrGet(&stype, OCI_ATTR_STMT_TYPE, xsink))
-	    return -1;
+         // see what kind of statement was prepared and set the flag accordingly
+         ub2 stype;
+         if (attrGet(&stype, OCI_ATTR_STMT_TYPE, xsink))
+            return -1;
 
-	 if (stype == OCI_STMT_SELECT)
-	    is_select = true;
+         if (stype == OCI_STMT_SELECT)
+            is_select = true;
       }
 
       return rc;
    }
 
-   DLLLOCAL int execute(ExceptionSink* xsink, const char* who);
-
    DLLLOCAL bool next(ExceptionSink* xsink) {
       return fetch(xsink) ? false : true;
    }
@@ -220,18 +208,16 @@ public:
    }
 
    DLLLOCAL QoreHashNode* fetchRow(OraResultSet& columns, ExceptionSink* xsink);
-#ifdef _QORE_HAS_DBI_SELECT_ROW
    DLLLOCAL QoreHashNode* fetchSingleRow(ExceptionSink* xsink);
-#endif
    DLLLOCAL QoreListNode* fetchRows(OraResultSet& columns, int rows, ExceptionSink* xsink);
    DLLLOCAL QoreListNode* fetchRows(ExceptionSink* xsink);
 
-   DLLLOCAL QoreHashNode* fetchColumns(OraResultSet& columns, int rows, ExceptionSink* xsink);
-   DLLLOCAL QoreHashNode* fetchColumns(ExceptionSink* xsink);
+   DLLLOCAL void doColumns(OraResultSet& columns, QoreHashNode& h);
+
+   DLLLOCAL QoreHashNode* fetchColumns(OraResultSet& columns, int rows, bool cols, ExceptionSink* xsink);
+   DLLLOCAL QoreHashNode* fetchColumns(bool cols, ExceptionSink* xsink);
 
-#ifdef _QORE_HAS_DBI_DESCRIBE
    DLLLOCAL QoreHashNode* describe(OraResultSet& columns, ExceptionSink* xsink);
-#endif
 
    DLLLOCAL Datasource* getDatasource() const {
       return ds;
