@@ -4,7 +4,7 @@
 
   Qore Programming Language
 
-  Copyright (C) 2006 - 2016 Qore Technologies, sro
+  Copyright (C) 2006 - 2017 Qore Technologies, s.r.o.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 #include "oracle.h"
 
 #include <stdlib.h>
+#include <memory>
 
 // OCI callback function for dynamic binds
 static sb4 ora_dynamic_bind_callback(void* ictxp, OCIBind* bindp, ub4 iter, ub4 index, void** bufpp, ub4* alenp, ub1* piecep, void** indp) {
@@ -49,6 +50,82 @@ static sb4 ora_dynamic_bind_placeholder_callback(void* ictxp, OCIBind* bindp, ub
    return OCI_CONTINUE;
 }
 
+int OraBindNode::setPlaceholder(const AbstractQoreNode* v, ExceptionSink* xsink) {
+   //printd(5, "OraBindNode::setPlaceholder() this: %p v: %p (%s)\n", this, v, get_type_name(v));
+   resetPlaceholder(xsink, false);
+
+   assert(!array);
+
+   // assume string if no argument passed
+   if (is_nothing(v)) {
+      setPlaceholderIntern(-1, "string", xsink);
+      return 0;
+   }
+
+   qore_type_t vtype = v->getType();
+   if (vtype == NT_HASH) {
+      const QoreHashNode* h = reinterpret_cast<const QoreHashNode*>(v);
+
+      int size = -1;
+      const AbstractQoreNode* t = h->getKeyValue("value");
+      if (t) {
+         assert(!value);
+         value = t->refSelf();
+      }
+      else {
+         // get and check size
+         const AbstractQoreNode* sz = h->getKeyValue("size");
+         size = sz ? sz->getAsInt() : -1;
+      }
+
+      // get and check data type
+      t = h->getKeyValue("type");
+      if (!t) {
+         if (value) {
+            setPlaceholderIntern(size, value->getTypeName(), xsink);
+            return 0;
+         }
+         xsink->raiseException("DBI-EXEC-EXCEPTION", "missing 'type' key in placeholder hash");
+         return -1;
+      }
+
+      if (t->getType() != NT_STRING) {
+         xsink->raiseException("DBI-EXEC-EXCEPTION", "expecting type name as value of 'type' key, got '%s'", t->getTypeName());
+         return -1;
+      }
+      const QoreStringNode* str = reinterpret_cast<const QoreStringNode*>(t);
+
+      //QoreStringValueHelper strdebug(v);
+      //printd(5, "OraBindNode::setPlaceholder() adding placeholder name=%s, size=%d, type=%s, value=%s\n", tstr.getBuffer(), size, str->getBuffer(), strdebug->getBuffer());
+      setPlaceholderIntern(size, str->getBuffer(), xsink);
+   }
+   else if (vtype == NT_STRING)
+      setPlaceholderIntern(-1, (reinterpret_cast<const QoreStringNode*>(v))->getBuffer(), xsink);
+   else if (vtype == NT_INT)
+      setPlaceholderIntern((reinterpret_cast<const QoreBigIntNode*>(v))->val, "string", xsink);
+   else {
+      xsink->raiseException("DBI-BIND-EXCEPTION", "expecting string or hash for placeholder description, got '%s'", v->getTypeName());
+      return -1;
+   }
+
+   return 0;
+}
+
+void OraBindNode::clearPlaceholder(ExceptionSink* xsink) {
+   if (array) {
+      delete buf.arraybind;
+      array = false;
+      return;
+   }
+
+   // free buffer data if any
+   del(xsink);
+
+   dtype = 0;
+   if (alt_output)
+      alt_output = false;
+}
+
 void OraBindNode::resetPlaceholder(ExceptionSink* xsink, bool free_name) {
    data.resetPlaceholder(free_name);
 
@@ -62,6 +139,8 @@ void OraBindNode::resetPlaceholder(ExceptionSink* xsink, bool free_name) {
    del(xsink);
 
    dtype = 0;
+   if (alt_output)
+      alt_output = false;
 }
 
 int OraBindNode::set(const AbstractQoreNode* v, ExceptionSink* xsink) {
@@ -74,10 +153,17 @@ int OraBindNode::set(const AbstractQoreNode* v, ExceptionSink* xsink) {
    return setPlaceholder(v, xsink);
 }
 
+void OraBindNode::clear(ExceptionSink* xsink, bool free_name) {
+   if (isValue())
+      resetValue(xsink);
+   else
+      clearPlaceholder(xsink);
+}
+
 void OraBindNode::reset(ExceptionSink* xsink, bool free_name) {
    if (value) {
       value->deref(xsink);
-      value = 0;
+      value = nullptr;
    }
 
    if (isValue())
@@ -97,15 +183,16 @@ int OraBindNode::bindDate(int pos, ExceptionSink* xsink) {
    return stmt.bindByPos(bndp, pos, &buf.odt, 0, QORE_SQLT_TIMESTAMP, xsink, pIndicator);
 }
 
-void OraBindNode::bind(int pos, bool& is_nty, ExceptionSink* xsink) {
+void OraBindNode::bind(int pos, ExceptionSink* xsink) {
+   //printd(5, "OBN::bind() pos: %d type: %s value: %p (%s)\n", pos, isValue() ? "value" : "placeholder", value, get_type_name(value));
    if (isValue()) {
-      bindValue(xsink, pos, value, is_nty);
+      bindValue(xsink, pos, value);
       return;
    }
 
    // TODO/FIXME: NTY is not possible to bind as IN OUT yet
    if (data.isType(ORACLE_OBJECT) || data.isType(ORACLE_COLLECTION)) {
-      bindPlaceholder(pos, is_nty, xsink);
+      bindPlaceholder(pos, xsink);
       return;
    }
 
@@ -118,41 +205,41 @@ void OraBindNode::bind(int pos, bool& is_nty, ExceptionSink* xsink) {
       // convert value to declared buffer type and then do bind with value
       if (data.isType("string") || data.isType("clob")) {
          QoreStringNodeValueHelper tmp(value);
-         bindValue(xsink, pos, *tmp, is_nty, false);
+         bindValue(xsink, pos, *tmp, false);
          //printd(5, "OraBindNode::bind(%d, str='%s' '%s')\n", pos, tmp->getBuffer(), value->getType() == NT_STRING ? reinterpret_cast<const QoreStringNode*>(value)->getBuffer() : value->getTypeName());
       }
       else if (data.isType("date")) {
          DateTimeNodeValueHelper tmp(value);
-         bindValue(xsink, pos, *tmp, is_nty, false);
+         bindValue(xsink, pos, *tmp, false);
       }
       else if (data.isType("binary") || data.isType("blob")) {
          if (value->getType() == NT_BINARY)
-            bindValue(xsink, pos, value, is_nty, false);
+            bindValue(xsink, pos, value, false);
          else {
             QoreStringNodeValueHelper tmp(value);
             if (!tmp->empty()) {
                SimpleRefHolder<BinaryNode> bt(new BinaryNode((void*)tmp->getBuffer(), tmp->size()));
-               bindValue(xsink, pos, *bt, is_nty, false);
+               bindValue(xsink, pos, *bt, false);
             }
             else {
-               bindPlaceholder(pos, is_nty, xsink);
+               bindPlaceholder(pos, xsink);
             }
          }
       }
       else if (data.isType("integer")) {
          if (value->getType() == NT_INT)
-            bindValue(xsink, pos, value, is_nty, false);
+            bindValue(xsink, pos, value, false);
          else {
             SimpleRefHolder<QoreBigIntNode> it(new QoreBigIntNode(value->getAsBigInt()));
-            bindValue(xsink, pos, *it, is_nty, false);
+            bindValue(xsink, pos, *it, false);
          }
       }
       else if (data.isType("float")) {
          if (value->getType() == NT_FLOAT)
-            bindValue(xsink, pos, value, is_nty, false);
+            bindValue(xsink, pos, value, false);
          else {
             SimpleRefHolder<QoreFloatNode> ft(new QoreFloatNode(value->getAsFloat()));
-            bindValue(xsink, pos, *ft, is_nty, false);
+            bindValue(xsink, pos, *ft, false);
          }
       }
       else {
@@ -161,7 +248,7 @@ void OraBindNode::bind(int pos, bool& is_nty, ExceptionSink* xsink) {
       return;
    }
 
-   bindPlaceholder(pos, is_nty, xsink);
+   bindPlaceholder(pos, xsink);
 }
 
 template <typename T>
@@ -288,7 +375,7 @@ public:
                max = nstr->size() + 1;
             alen_list[li.index()] = nstr->size() + 1;
 
-            std::auto_ptr<QoreString> tstr(nstr.giveString());
+            std::unique_ptr<QoreString> tstr(nstr.giveString());
             strvec.setDynamic(tstr->giveBuffer());
          }
       }
@@ -407,7 +494,7 @@ public:
          }
 
          if (t != NT_FLOAT) {
-            xsink->raiseException("ARRAY-BIND-ERROR", "found type \"%s\" in list element "QLLD" (starting from 0) expecting type \"float\"; all list elements must be of the same type to effect an array bind", get_type_name(n), li.index());
+            xsink->raiseException("ARRAY-BIND-ERROR", "found type \"%s\" in list element " QLLD " (starting from 0) expecting type \"float\"; all list elements must be of the same type to effect an array bind", get_type_name(n), li.index());
             return -1;
          }
 
@@ -491,7 +578,7 @@ public:
          }
 
          if (t != NT_DATE) {
-            xsink->raiseException("ARRAY-BIND-ERROR", "found type \"%s\" in list element "QLLD" (starting from 0) expecting type \"date\"; all list elements must be of the same type to effect an array bind", get_type_name(n), li.index());
+            xsink->raiseException("ARRAY-BIND-ERROR", "found type \"%s\" in list element " QLLD " (starting from 0) expecting type \"date\"; all list elements must be of the same type to effect an array bind", get_type_name(n), li.index());
             return -1;
          }
 
@@ -565,7 +652,7 @@ public:
          }
 
          if (t != NT_BOOLEAN) {
-            xsink->raiseException("ARRAY-BIND-ERROR", "found type \"%s\" in list element "QLLD" (starting from 0) expecting type \"boolean\"; all list elements must be of the same type to effect an array bind", get_type_name(n), li.index());
+            xsink->raiseException("ARRAY-BIND-ERROR", "found type \"%s\" in list element " QLLD " (starting from 0) expecting type \"boolean\"; all list elements must be of the same type to effect an array bind", get_type_name(n), li.index());
             return -1;
          }
 
@@ -641,7 +728,7 @@ public:
          }
 
          if (t != NT_BINARY) {
-            xsink->raiseException("ARRAY-BIND-ERROR", "found type \"%s\" in list element "QLLD" (starting from 0) expecting type \"binary\"; all list elements must be of the same type to effect an array bind", get_type_name(n), li.index());
+            xsink->raiseException("ARRAY-BIND-ERROR", "found type \"%s\" in list element " QLLD " (starting from 0) expecting type \"binary\"; all list elements must be of the same type to effect an array bind", get_type_name(n), li.index());
             return -1;
          }
 
@@ -759,7 +846,7 @@ public:
          }
 
          if (t != NT_BINARY) {
-            xsink->raiseException("ARRAY-BIND-ERROR", "found type \"%s\" in list element "QLLD" (starting from 0) expecting type \"binary\"; all list elements must be of the same type to effect an array bind", get_type_name(n), ind);
+            xsink->raiseException("ARRAY-BIND-ERROR", "found type \"%s\" in list element " QLLD " (starting from 0) expecting type \"binary\"; all list elements must be of the same type to effect an array bind", get_type_name(n), ind);
             return -1;
          }
 
@@ -1208,7 +1295,7 @@ void OraBindNode::bindListValue(ExceptionSink* xsink, int pos, const AbstractQor
    //printd(5, "OraBindNode::bindListValue() OCIBindDynamic t: %d\n", t);
 }
 
-void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNode* v, bool& is_nty, bool in_only) {
+void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNode* v, bool in_only) {
    QoreOracleConnection* conn = stmt.getData();
 
    ind = 0;
@@ -1262,7 +1349,7 @@ void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNod
 
       // bind it
       if ((len + 1) > LOB_THRESHOLD && in_only) {
-         //printd(5, "binding string %p len: %lld as CLOB\n", buf.ptr, len);
+         //printd(5, "binding string %p len: " QLLD " as CLOB\n", buf.ptr, len);
          // bind as a CLOB
          dtype = SQLT_CLOB;
 
@@ -1313,7 +1400,7 @@ void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNod
       qore_size_t len = b->size();
 
       if (len > LOB_THRESHOLD && in_only) {
-         //printd(5, "binding binary %p len: %lld as BLOB\n", buf.ptr, len);
+         //printd(5, "binding binary %p len: " QLLD " as BLOB\n", buf.ptr, len);
          // bind as a BLOB
          dtype = SQLT_BLOB;
 
@@ -1339,7 +1426,7 @@ void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNod
       else {
          dtype = SQLT_BIN;
 
-         //printd(5, "OraBindNode::bindValue() BLOB ptr: %p size: "QLLD"\n", b->getPtr(), b->size());
+         //printd(5, "OraBindNode::bindValue() BLOB ptr: %p size: " QLLD "\n", b->getPtr(), b->size());
 
          if (!in_only) {
             // bind a copy of the value in case of in/out variables
@@ -1375,7 +1462,7 @@ void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNod
          dtype = SQLT_STR;
 
          QoreString* tstr = new QoreString(stmt.getEncoding());
-         tstr->sprintf("%lld", b->val);
+         tstr->sprintf(QLLD, b->val);
          data.save(tstr);
 
          //printd(5, "binding number '%s'\n", buf.ptr);
@@ -1384,7 +1471,6 @@ void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNod
       return;
    }
 
-#ifdef _QORE_HAS_NUMBER_TYPE
    if (ntype == NT_NUMBER) {
       const QoreNumberNode* n = reinterpret_cast<const QoreNumberNode*>(v);
       // bind as a string value
@@ -1398,7 +1484,6 @@ void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNod
       stmt.bindByPos(bndp, pos, (char* )tstr->getBuffer(), tstr->strlen() + 1, SQLT_STR, xsink, pIndicator);
       return;
    }
-#endif
 
    if (ntype == NT_FLOAT) {
 #if defined(SQLT_BDOUBLE) && defined(USE_NEW_NUMERIC_TYPES)
@@ -1423,7 +1508,6 @@ void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNod
       const QoreStringNode*  t = reinterpret_cast<const QoreStringNode*>(h->getKeyValue("type"));
       if (t->compare(ORACLE_OBJECT) == 0) {
          //printd(5, "binding hash as an oracle object\n");
-         is_nty = true;
          subdtype = SQLT_NTY_OBJECT;
          dtype = SQLT_NTY;
          buf.oraObj = objBindQore(conn, h, xsink);
@@ -1442,7 +1526,6 @@ void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNod
       }
       else if (t->compare(ORACLE_COLLECTION) == 0) {
          //printd(5, "binding list as an oracle collection\n");
-         is_nty = true;
          subdtype = SQLT_NTY_COLLECTION;
          dtype = SQLT_NTY;
          buf.oraColl = collBindQore(conn, h, xsink);
@@ -1469,7 +1552,7 @@ void OraBindNode::bindValue(ExceptionSink* xsink, int pos, const AbstractQoreNod
    xsink->raiseException("ORACLE-BIND-VALUE-ERROR", "type '%s' is not supported for SQL binding", v->getTypeName());
 }
 
-void OraBindNode::bindPlaceholder(int pos, bool& is_nty, ExceptionSink* xsink) {
+void OraBindNode::bindPlaceholder(int pos, ExceptionSink* xsink) {
    QoreOracleConnection* conn = stmt.getData();
 
    //printd(5, "OraBindNode::bindPlaceholder() this: %p, conn: %p, pos: %d type: %s, size: %d buf.ptr: %p\n", this, conn, pos, data.ph_type, data.ph_maxsize, buf.ptr);
@@ -1564,7 +1647,7 @@ void OraBindNode::bindPlaceholder(int pos, bool& is_nty, ExceptionSink* xsink) {
             const BinaryNode* bin = reinterpret_cast<const BinaryNode*>(value);
             // if too big, raise an exception (not likely to happen)
             if (bin->size() > 0x7fffffff) {
-               xsink->raiseException("BIND-ERROR", "value passed for binding is %lld bytes long, which is too big to bind as a long binary value, maximum size is %d bytes", bin->size(), 0x7fffffff);
+               xsink->raiseException("BIND-ERROR", "value passed for binding is " QLLD " bytes long, which is too big to bind as a long binary value, maximum size is %d bytes", bin->size(), 0x7fffffff);
                return;
             }
             size_t size = bin->size() + sizeof(int);
@@ -1583,7 +1666,7 @@ void OraBindNode::bindPlaceholder(int pos, bool& is_nty, ExceptionSink* xsink) {
                return;
             // if too big, raise an exception (not likely to happen)
             if (str->strlen() > 0x7fffffff) {
-               xsink->raiseException("BIND-ERROR", "value passed for binding is %lld bytes long, which is too big to bind as a long binary value, maximum size is %d bytes", str->strlen(), 0x7fffffff);
+               xsink->raiseException("BIND-ERROR", "value passed for binding is " QLLD " bytes long, which is too big to bind as a long binary value, maximum size is %d bytes", str->strlen(), 0x7fffffff);
                return;
             }
             size_t size = str->strlen() + sizeof(int);
@@ -1659,12 +1742,12 @@ void OraBindNode::bindPlaceholder(int pos, bool& is_nty, ExceptionSink* xsink) {
          if (size < 100)
             size = 100;
          buf.ptr = malloc(sizeof(char) * (size + 1));
-         strcpy((char* )buf.ptr, str->getBuffer());
+         strcpy((char*)buf.ptr, str->getBuffer());
       }
       else {
          size = 100;
          buf.ptr = malloc(sizeof(char) * (size + 1));
-         ((char* )buf.ptr)[0] = '\0';
+         ((char*)buf.ptr)[0] = '\0';
       }
 
       // we actually bind as a string
@@ -1675,13 +1758,22 @@ void OraBindNode::bindPlaceholder(int pos, bool& is_nty, ExceptionSink* xsink) {
 
       // allocate statement handle for result list
       if (conn->handleAlloc(&buf.ptr, OCI_HTYPE_STMT, "OraBindNode::bindPlaceHolder() allocate statement handle", xsink))
-         buf.ptr = 0;
+         buf.ptr = nullptr;
       else
          stmt.bindByPos(bndp, pos, &buf.ptr, 0, SQLT_RSET, xsink, &ind);
    }
+   else if (data.isType("resultset")) {
+      dtype = SQLT_RSET;
+
+      // allocate statement handle for result list
+      if (conn->handleAlloc(&buf.ptr, OCI_HTYPE_STMT, "OraBindNode::bindPlaceHolder() allocate statement handle", xsink))
+         buf.ptr = nullptr;
+      else {
+         stmt.bindByPos(bndp, pos, &buf.ptr, 0, SQLT_RSET, xsink, &ind);
+         alt_output = true;
+      }
+   }
    else if (data.isType(ORACLE_OBJECT)) {
-      if (!is_nty)
-         is_nty = true;
       subdtype = SQLT_NTY_OBJECT;
       dtype = SQLT_NTY;
 
@@ -1690,7 +1782,7 @@ void OraBindNode::bindPlaceholder(int pos, bool& is_nty, ExceptionSink* xsink) {
        if (h->existsKey("^values^"))
            buf.oraObj = objBindQore(conn, h, xsink); // IN/OUT
        else {
-           const QoreStringNode*  str = reinterpret_cast<const QoreStringNode*>(h->getKeyValue("value"));
+           const QoreStringNode* str = reinterpret_cast<const QoreStringNode*>(h->getKeyValue("value"));
            buf.oraObj = objPlaceholderQore(conn, str->getBuffer(), xsink); // IN
        }
 #endif
@@ -1727,8 +1819,6 @@ void OraBindNode::bindPlaceholder(int pos, bool& is_nty, ExceptionSink* xsink) {
        //printd(5, "OraBindNode::bindValue() object '%s' tdo=0x%x handle=0x%x\n", h->getBuffer(), buf.oraObj->typinf->tdo, buf.oraObj->handle);
    }
    else if (data.isType(ORACLE_COLLECTION)) {
-      if (!is_nty)
-         is_nty = true;
       subdtype = SQLT_NTY_COLLECTION;
       dtype = SQLT_NTY;
 
@@ -1818,22 +1908,134 @@ AbstractQoreNode* OraBindNode::getValue(bool horizontal, ExceptionSink* xsink) {
    return OraColumnValue::getValue(xsink, horizontal, true);
 }
 
+int QorePreparedStatement::execute(ExceptionSink* xsink, const char* who, int oci_flags) {
+   assert(conn.svchp);
+   ub4 iters;
+   if (is_select)
+      iters = 0;
+   else
+      iters = !array_size ? 1 : array_size;
+   int status = OCIStmtExecute(conn.svchp, stmthp, conn.errhp, iters, 0, 0, 0, OCI_DEFAULT | oci_flags);
+
+   //printd(5, "QoreOracleStatement::execute() stmthp: %p status: %d (OCI_ERROR: %d)\n", stmthp, status, OCI_ERROR);
+   if (status == OCI_ERROR) {
+      // see if we have a lost connection from the error code
+      int ping = -1;
+
+      // get and save error information
+      sb4 errcode;
+      text errbuf[512];
+      OCIErrorGet((dvoid*)conn.errhp, (ub4)1, (text*)nullptr, &errcode, errbuf, (ub4)sizeof(errbuf), OCI_HTYPE_ERROR);
+      // ORA-03113 and ORA-03114 are lost connections
+      // issue #802: ORA-01041 is only raised due to a bug in Oracle pre 12c
+      if (errcode == 3113 || errcode == 3114 || errcode == 1041)
+         ping = 0;
+
+      //dbg();
+      // see if server is connected
+      if (ping == -1) {
+         ExceptionSink xsink2;
+         ping = OCI_Ping(&conn.ocilib, conn.ocilib_cn, &xsink2);
+         // do not allow ping exceptions to be propagated to the caller
+         xsink2.clear();
+      }
+
+      if (!ping) {
+         // if there is at least one SQLStatement active on the connection, there will be a transaction in place
+         // therefore in all cases all statements will be invalidated and closed when we lose a connection
+         // we can only recover a plain "exec" or "select" call
+         if (ds->activeTransaction())
+            xsink->raiseException("DBI:ORACLE:TRANSACTION-ERROR", "connection to Oracle database server %s@%s lost while in a transaction; transaction has been lost", ds->getUsername(), ds->getDBName());
+
+         // reset current statement state while the driver-specific context data is still present
+         clear(xsink);
+         // free and reset statement states for all active statements while the driver-specific context data is still present
+         ds->connectionLost(xsink);
+
+         // try to reconnect
+         conn.logoff();
+
+         //printd(5, "QoreOracleStatement::execute() about to execute OCILogon() for reconnect (trans: %d)\n", ds->activeTransaction());
+         if (conn.logon(xsink)) {
+            //printd(5, "QoreOracleStatement::execute() conn: %p reconnect failed, marking connection as closed\n", &conn);
+            // free state completely
+            reset(xsink);
+            // close datasource and remove private data
+            ds->connectionAborted(xsink);
+            return -1;
+         }
+
+         // clear warnings
+         conn.clearWarnings();
+
+         // don't execute again if any exceptions have occured, including if the connection was aborted while in a transaction
+         if (*xsink) {
+            // close all statements and remove private data but leave datasource open
+            ds->connectionRecovered(xsink);
+            return -1;
+         }
+
+         // try to recreate the statement context
+         if (rebindAbortedConnection(xsink))
+            return -1;
+
+         assert(!*xsink);
+
+         //printd(5, "QoreOracleStatement::execute() returned from OCILogon() status: %d\n", status);
+         status = OCIStmtExecute(conn.svchp, stmthp, conn.errhp, iters, 0, 0, 0, OCI_DEFAULT | oci_flags);
+         if (status && conn.checkerr(status, who, xsink))
+            return -1;
+      }
+      else {
+         //printd(5, "QoreOracleStatement::execute() error, but it's connected; status: %d who: %s\n", status, who);
+         conn.doException(who, errbuf, errcode, xsink);
+         //conn.checkerr(status, who, xsink);
+         return -1;
+      }
+   }
+   else if (status && conn.checkerr(status, who, xsink))
+      return -1;
+
+   return 0;
+}
+
 int QorePreparedStatement::exec(ExceptionSink* xsink) {
    return execute(xsink, "QorePreparedStatement::exec()");
 }
 
-void QorePreparedStatement::reset(ExceptionSink* xsink) {
-   //printd(5, "QorePreparedStatement::reset() this: %p\n", this);
+int QorePreparedStatement::execDescribe(ExceptionSink* xsink) {
+    //printd(5, "QorePreparedStatement::execDescribe() this: %p\n", this);
+    return execute(xsink, "QorePreparedStatement::execDescribe()", OCI_DESCRIBE_ONLY);
+}
+
+void QorePreparedStatement::clear(ExceptionSink* xsink) {
+   //printd(5, "QorePreparedStatement::clear() this: %p\n", this);
 
    QoreOracleStatement::reset(xsink);
 
-   delete str;
-   str = 0;
+   // clear all nodes without deleting the values
+   for (node_list_t::iterator i = node_list.begin(), e = node_list.end(); i != e; ++i) {
+      (*i)->clear(xsink);
+   }
+
+   if (columns) {
+      columns->del(xsink);
+      delete columns;
+      columns = nullptr;
+      array_size = 0;
+   }
+
+   defined = false;
+}
+
+void QorePreparedStatement::reset(ExceptionSink* xsink) {
+   QoreOracleStatement::reset(xsink);
 
    for (node_list_t::iterator i = node_list.begin(), e = node_list.end(); i != e; ++i) {
       (*i)->reset(xsink);
       delete *i;
    }
+
    node_list.clear();
 
    if (columns) {
@@ -1843,9 +2045,18 @@ void QorePreparedStatement::reset(ExceptionSink* xsink) {
       array_size = 0;
    }
 
-   hasOutput = false;
    defined = false;
-   has_nty = false;
+   hasOutput = false;
+
+   if (str) {
+      delete str;
+      str = 0;
+   }
+
+   if (args_copy) {
+      args_copy->deref(xsink);
+      args_copy = 0;
+   }
 }
 
 int QorePreparedStatement::define(ExceptionSink* xsink) {
@@ -1866,6 +2077,7 @@ int QorePreparedStatement::define(ExceptionSink* xsink) {
 }
 
 int QorePreparedStatement::prepare(const QoreString& sql, const QoreListNode* args, bool parse, ExceptionSink* xsink) {
+   assert(!str);
    // create copy of string and convert encoding if necessary
    str = sql.convertEncoding(getEncoding(), xsink);
    if (*xsink)
@@ -1886,10 +2098,8 @@ int QorePreparedStatement::prepare(const QoreString& sql, const QoreListNode* ar
    if (QoreOracleStatement::prepare(*str, xsink))
       return -1;
 
-   if (!node_list.empty()) {
-      if (bindOracle(xsink))
-         return -1;
-   }
+   if (!node_list.empty() && bindOracle(xsink))
+      return -1;
 
    return 0;
 }
@@ -1913,7 +2123,7 @@ int QorePreparedStatement::bindOracle(ExceptionSink* xsink) {
 
    int pos = 1;
    for (node_list_t::iterator i = node_list.begin(), e = node_list.end(); i != e; ++i) {
-      (*i)->bind(pos, has_nty, xsink);
+      (*i)->bind(pos, xsink);
 
       if (*xsink)
          return -1;
@@ -2113,6 +2323,13 @@ void QorePreparedStatement::parseQuery(const QoreListNode* args, ExceptionSink*
 
       ++p;
    }
+
+   // save args if appropriate
+   if (args && !ds->isInTransaction() && !*xsink) {
+      if (args_copy)
+         args_copy->deref(xsink);
+      args_copy = args->listRefSelf();
+   }
 }
 
 QoreHashNode* QorePreparedStatement::getOutputHash(bool rows, ExceptionSink* xsink) {
@@ -2127,10 +2344,9 @@ QoreHashNode* QorePreparedStatement::getOutputHash(bool rows, ExceptionSink* xsi
    return *xsink ? 0 : h.release();
 }
 
-#ifdef _QORE_HAS_DBI_SELECT_ROW
 QoreHashNode* QorePreparedStatement::selectRow(ExceptionSink* xsink) {
    if (!is_select) {
-      xsink->raiseException("ORACLE-SELECT-ROW-ERROR", "the SQL passed to the selectRow() method is not a select statement");
+      xsink->raiseException("DBI-SELECT-ROW-ERROR", "the SQL passed to the selectRow() method is not a select statement");
       return 0;
    }
 
@@ -2139,9 +2355,8 @@ QoreHashNode* QorePreparedStatement::selectRow(ExceptionSink* xsink) {
 
    return fetchSingleRow(xsink);
 }
-#endif
 
-AbstractQoreNode* QorePreparedStatement::execWithPrologue(bool rows, ExceptionSink* xsink) {
+AbstractQoreNode* QorePreparedStatement::execWithPrologue(ExceptionSink* xsink, bool rows, bool cols) {
    if (exec(xsink))
       return 0;
 
@@ -2152,7 +2367,7 @@ AbstractQoreNode* QorePreparedStatement::execWithPrologue(bool rows, ExceptionSi
       if (rows)
          rv = QoreOracleStatement::fetchRows(xsink);
       else
-         rv = QoreOracleStatement::fetchColumns(xsink);
+         rv = QoreOracleStatement::fetchColumns(cols, xsink);
 
       if (*xsink)
          return 0;
@@ -2189,12 +2404,10 @@ QoreListNode* QorePreparedStatement::fetchRows(int rows, ExceptionSink* xsink) {
 
 QoreHashNode* QorePreparedStatement::fetchColumns(int rows, ExceptionSink* xsink) {
    assert(columns);
-   return QoreOracleStatement::fetchColumns(*columns, rows, xsink);
+   return QoreOracleStatement::fetchColumns(*columns, rows, false, xsink);
 }
 
-#ifdef _QORE_HAS_DBI_DESCRIBE
 QoreHashNode* QorePreparedStatement::describe(ExceptionSink* xsink) {
    assert(columns);
    return QoreOracleStatement::describe(*columns, xsink);
 }
-#endif
