@@ -65,6 +65,19 @@
             (res) = TRUE;                                                      \
     }
 
+#define OCI_CALL0Q(ocilib, res, err, fct, xsink)				\
+                                                                               \
+    {                                                                          \
+        (res) = (boolean) fct;                                                 \
+        if (OCI_NO_ERROR((res)) == FALSE)                                      \
+        {                                                                      \
+            (res) = ((res) == OCI_SUCCESS_WITH_INFO);                          \
+            OCI_ExceptionOCI2(ocilib, (err), NULL, NULL, res, xsink);		\
+        }                                                                      \
+        else                                                                   \
+            (res) = TRUE;                                                      \
+    }
+
 /**
  * @brief 
  * Conditional OCI call with return value checking
@@ -97,6 +110,22 @@
         }                                                                      \
     }
 
+#define OCI_CALL1Q(ocilib, res, con, stmt, fct, xsink)			\
+                                                                               \
+    {                                                                          \
+        if ((res) == TRUE)                                                     \
+        {                                                                      \
+            (res) = (boolean) fct;                                             \
+            if (OCI_NO_ERROR((res)) == FALSE)                                  \
+            {                                                                  \
+                (res) = ((res) == OCI_SUCCESS_WITH_INFO);                      \
+		OCI_ExceptionOCI2(ocilib, (con)->err, (con), (stmt), res, xsink); \
+            }                                                                  \
+            else                                                               \
+                (res) = TRUE;                                                  \
+        }                                                                      \
+    }
+
 /**
  * @brief 
  * Conditional OCI call with return value checking
@@ -220,6 +249,22 @@
         }                                                                      \
     }
 
+#define OCI_CALL4Q(ocilib, res, err, con, fct, xsink)                          \
+                                                                               \
+    {                                                                          \
+        if ((res) == TRUE)                                                     \
+        {                                                                      \
+            (res) = (boolean) fct;                                             \
+            if (OCI_NO_ERROR((res)) == FALSE)                                  \
+            {                                                                  \
+                (res) = ((res) == OCI_SUCCESS_WITH_INFO);                      \
+                OCI_ExceptionOCI2(ocilib, (err), (con), NULL, res, xsink);     \
+            }                                                                  \
+            else                                                               \
+                (res) = TRUE;                                                  \
+        }                                                                      \
+    }
+
 /**
  * @brief 
  * Direct OCI call with return value checking
@@ -280,6 +325,9 @@
  *
  */
 
+#define OCI_CHECK_PTR(ocilib, type, ptr, ret) assert(ptr)
+
+/*
 #define OCI_CHECK_PTR(ocilib, type, ptr, ret)				\
                                                                                \
     if ((ptr) == NULL)                                                         \
@@ -288,6 +336,16 @@
                                                                                \
         return (ret);                                                          \
     }           
+*/
+
+#define OCI_CHECK_PTRQ(ocilib, type, ptr, ret, xsink)				\
+                                                                               \
+    if ((ptr) == NULL)                                                         \
+    {                                                                          \
+       OCI_ExceptionNullPointer2(ocilib, type, xsink);			\
+                                                                               \
+        return (ret);                                                          \
+    }           
 
 /**
  * @brief 
@@ -357,6 +415,15 @@
         return (ret);                                                          \
     } 
 
+#define OCI_CHECK_BOUNDQ(ocilib, con, v, b1, b2, ret, xsink)			\
+                                                                               \
+    if ((v < (b1)) || (v > (b2)))                                              \
+    {                                                                          \
+       OCI_ExceptionOutOfBounds2(ocilib, (con), (v), xsink);			\
+                                                                               \
+        return (ret);                                                          \
+    } 
+
 /**
  * @brief 
  * Checks if an integer parameter value is >= minimum provided value
@@ -372,13 +439,13 @@
  *
  */
 
-#define OCI_CHECK_MIN(ocilib, con, stmt, v, m, ret)			\
+#define OCI_CHECK_MIN(ocilib, con, stmt, v, m, ret, xsink)			\
                                                                                \
     if ((v) < (m))                                                             \
     {                                                                          \
-       OCI_ExceptionMinimumValue2(ocilib, (con), (stmt), m);		\
+       OCI_ExceptionMinimumValue2(ocilib, (con), (stmt), m, xsink);		\
                                                                                \
-        return (ret);                                                          \
+       return (ret);							\
     } 
 
 /**
@@ -394,16 +461,17 @@
  *
  */
 
-#define OCI_CHECK_COMPAT(ocilib, con, exp, ret)				\
+#define OCI_CHECK_COMPAT(ocilib, con, exp, ret) assert(exp)
+
+#define OCI_CHECK_COMPATQ(ocilib, con, exp, ret, xsink)			\
                                                                                \
     if ((exp) == FALSE)                                                         \
     {                                                                          \
-       OCI_ExceptionTypeNotCompatible2(ocilib, (con));			\
+       OCI_ExceptionTypeNotCompatible2(ocilib, (con), xsink);			\
                                                                                \
         return (ret);                                                          \
     } 
 
-
 /* ************************************************************************ *
                   INTERNAL STATES/ATTRIBUTES CHECKING MACROS
  * ************************************************************************ */
@@ -505,6 +573,9 @@
  *
  */
 
+#define OCI_CHECK_INITIALIZED2(ocilib, ret) assert(ocilib->loaded)
+
+/*
 #define OCI_CHECK_INITIALIZED2(ocilib, ret)				\
                                                                                \
         if (ocilib->loaded == FALSE)                                            \
@@ -512,6 +583,7 @@
             OCI_ExceptionNotInitialized2(ocilib);                                     \
             return ret;                                                        \
         }
+*/
 
 /**
  * @brief 
@@ -527,11 +599,11 @@
  *
  */
 
-#define OCI_CHECK_FEATURE(ocilib, con, feat, ver,  ret)			\
+#define OCI_CHECK_FEATURE(ocilib, con, feat, ver, ret, xsink)			\
                                                                                    \
     if (ocilib->version_runtime < ver || (((con) != NULL) && (con)->ver_num < ver)) \
     {                                                                              \
-       OCI_ExceptionNotAvailable2(ocilib, con, feat);			\
+       OCI_ExceptionNotAvailable2(ocilib, con, feat, xsink);			\
         return ret;                                                                \
     }
 
@@ -568,9 +640,9 @@
  *
  */
 
-#define OCI_CHECK_TIMESTAMP_ENABLED(ocilib, con,  ret)			\
+#define OCI_CHECK_TIMESTAMP_ENABLED(ocilib, con, ret, xsink)			\
                                                                                \
-   OCI_CHECK_FEATURE(ocilib, con, OCI_FEATURE_TIMESTAMP, OCI_9_0, ret)
+    OCI_CHECK_FEATURE(ocilib, con, OCI_FEATURE_TIMESTAMP, OCI_9_0, ret, xsink)
 
 /**
  * @brief 
@@ -655,11 +727,11 @@
  *
  */
 
-#define OCI_CHECK_DATABASE_NOTIFY_ENABLED(ocilib, ret)			\
+#define OCI_CHECK_DATABASE_NOTIFY_ENABLED(ocilib, ret, xsink)			\
                                                                                \
     if (ocilib->version_runtime < OCI_10_2)                                     \
     {                                                                          \
-       OCI_ExceptionNotAvailable2(ocilib, NULL, OCI_FEATURE_DATABASE_NOTIFY); \
+       OCI_ExceptionNotAvailable2(ocilib, NULL, OCI_FEATURE_DATABASE_NOTIFY, xsink); \
         return ret;                                                            \
     }
 
