@@ -40,6 +40,9 @@
 
 #include <qore/Qore.h>
 
+#define NUM_FMT "9999999999999999999999999999999999.99999999999999999999999"
+#define NUM_FMT_LEN (sizeof(NUM_FMT)-1)
+
 /* ************************************************************************ *
  *                             PRIVATE TYPES
  * ************************************************************************ */
@@ -78,10 +81,13 @@ typedef struct OCI_Item OCI_Item;
 
 struct OCI_List
 {
-    OCI_Item    *head;   /* pointer to first item */
-    OCI_Mutex   *mutex;  /* mutex handle */
-    ub4          count;  /* number of elements in list */
-    int          type;   /* type of list item */
+   OCI_Item    *head;   /* pointer to first item */
+   QoreThreadLock mutex;  // mutex
+   ub4          count;  /* number of elements in list */
+   int          type;   /* type of list item */
+
+   DLLLOCAL OCI_List(int t) : head(0), count(0), type(t) {
+   }
 };
 
 typedef struct OCI_List OCI_List;
@@ -896,112 +902,113 @@ extern OCI_SQLCmdInfo SQLCmds[];
 
 OCI_EXPORT OCI_Error * OCI_API OCI_GetLastError2(OCI_Library *pOCILib);
 OCI_EXPORT boolean OCI_API OCI_Initialize2(OCI_Library *pOCILib, OCIEnv * d_ora_env, OCIError *errhp, POCI_ERROR err_handler, const mtext *lib_path, unsigned int mode);
-OCI_EXPORT boolean OCI_API OCI_Cleanup2(OCI_Library *pOCILib);
-OCI_EXPORT OCI_TypeInfo * OCI_API OCI_TypeInfoGet2(OCI_Library *pOCILib, OCI_Connection *con, const mtext *name, unsigned int type);
-OCI_EXPORT boolean OCI_API OCI_ObjectSetRef2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Ref *value);
-OCI_EXPORT boolean OCI_API OCI_ObjectGetSelfRef2(OCI_Library *pOCILib, OCI_Object *obj, OCI_Ref *ref);
+OCI_EXPORT boolean OCI_API OCI_Cleanup2(OCI_Library *pOCILib, ExceptionSink* xsink);
+OCI_EXPORT OCI_TypeInfo * OCI_API OCI_TypeInfoGet2(OCI_Library *pOCILib, OCI_Connection *con, const mtext *name, unsigned int type, ExceptionSink* xsink);
+//OCI_EXPORT boolean OCI_API OCI_ObjectSetRef2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Ref *value);
+//OCI_EXPORT boolean OCI_API OCI_ObjectGetSelfRef2(OCI_Library *pOCILib, OCI_Object *obj, OCI_Ref *ref);
 OCI_EXPORT boolean OCI_API OCI_ObjectSetObject2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Object *value, ExceptionSink* xsink);
 OCI_EXPORT boolean OCI_API OCI_ObjectSetColl2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Coll *value, ExceptionSink* xsink);
-OCI_EXPORT boolean OCI_API OCI_ObjectSetInterval2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Interval *value);
+OCI_EXPORT boolean OCI_API OCI_ObjectSetInterval2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Interval *value, ExceptionSink* xsink);
 OCI_EXPORT OCI_Object * OCI_API OCI_ObjectCreate2(OCI_Library *pOCILib, OCI_Connection *con, OCI_TypeInfo *typinf, ExceptionSink* xsink);
 OCI_EXPORT OCI_Object * OCI_API OCI_ObjectGetObject2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
-OCI_EXPORT boolean OCI_API OCI_ObjectFree2(OCI_Library *pOCILib, OCI_Object *obj);
-OCI_EXPORT boolean OCI_API OCI_ObjectAssign2(OCI_Library *pOCILib, OCI_Object *obj, OCI_Object *obj_src);
-OCI_EXPORT int OCI_API OCI_ObjectGetRaw2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, void *buffer, unsigned int len);
-OCI_EXPORT boolean OCI_API OCI_ObjectSetRaw2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, void* value, unsigned int len);
-OCI_EXPORT boolean OCI_API OCI_ObjectSetTimestamp2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Timestamp *value);
+OCI_EXPORT boolean OCI_API OCI_ObjectFree2(OCI_Library *pOCILib, OCI_Object *obj, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_ObjectAssign2(OCI_Library *pOCILib, OCI_Object *obj, OCI_Object *obj_src, ExceptionSink* xsink);
+//OCI_EXPORT int OCI_API OCI_ObjectGetRaw2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, void *buffer, unsigned int len);
+//OCI_EXPORT boolean OCI_API OCI_ObjectSetRaw2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, void* value, unsigned int len);
+OCI_EXPORT boolean OCI_API OCI_ObjectSetTimestamp2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Timestamp *value, ExceptionSink* xsink);
 OCI_EXPORT OCI_Object * OCI_API OCI_ElemGetObject2(OCI_Library * pOCILib, OCI_Elem *elem, ExceptionSink* xsink);
 OCI_EXPORT boolean OCI_API OCI_ElemSetObject2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Object *value, ExceptionSink* xsink);
-OCI_EXPORT unsigned int OCI_API OCI_ElemGetRaw2(OCI_Library *pOCILib, OCI_Elem *elem, void *value, unsigned int len);
-OCI_EXPORT boolean OCI_API OCI_ElemSetRaw2(OCI_Library *pOCILib, OCI_Elem *elem, void* value, unsigned int len);
+//OCI_EXPORT unsigned int OCI_API OCI_ElemGetRaw2(OCI_Library *pOCILib, OCI_Elem *elem, void *value, unsigned int len);
+//OCI_EXPORT boolean OCI_API OCI_ElemSetRaw2(OCI_Library *pOCILib, OCI_Elem *elem, void* value, unsigned int len, ExceptionSink* xsink);
 OCI_EXPORT const dtext * OCI_API OCI_ElemGetString2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT boolean OCI_API OCI_ElemSetString2(OCI_Library *pOCILib, OCI_Elem *elem, const dtext *value);
+OCI_EXPORT boolean OCI_API OCI_ElemSetString2(OCI_Library *pOCILib, OCI_Elem *elem, const dtext *value, ExceptionSink* xsink);
 OCI_EXPORT const dtext * OCI_API OCI_ObjectGetString2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
-OCI_EXPORT boolean OCI_API OCI_ObjectSetString2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, const dtext *value);
-OCI_EXPORT OCI_Lob * OCI_API OCI_ObjectGetLob2(OCI_Library * pOCILib, OCI_Object *obj, const mtext *attr);
-OCI_EXPORT OCI_Lob * OCI_API OCI_LobCreate2(OCI_Library *pOCILib, OCI_Connection *con, unsigned int type);
-OCI_EXPORT OCI_Lob * OCI_API OCI_ElemGetLob2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT boolean OCI_API OCI_ElemSetLob2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Lob *value);
+OCI_EXPORT boolean OCI_API OCI_ObjectSetString2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, const dtext *value, ExceptionSink* xsink);
+OCI_EXPORT OCI_Lob * OCI_API OCI_ObjectGetLob2(OCI_Library * pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
+OCI_EXPORT OCI_Lob * OCI_API OCI_LobCreate2(OCI_Library *pOCILib, OCI_Connection *con, unsigned int type, ExceptionSink* xsink);
+OCI_EXPORT OCI_Lob * OCI_API OCI_ElemGetLob2(OCI_Library *pOCILib, OCI_Elem *elem, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_ElemSetLob2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Lob *value, ExceptionSink* xsink);
 OCI_EXPORT OCI_Coll * OCI_API OCI_CollCreate2(OCI_Library *pOCILib, OCI_TypeInfo *typinf, ExceptionSink* xsink);
 OCI_EXPORT OCI_Coll * OCI_API OCI_ObjectGetColl2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
 OCI_EXPORT OCI_Coll * OCI_API OCI_ElemGetColl2(OCI_Library *pOCILib, OCI_Elem *elem, ExceptionSink* xsink);
 OCI_EXPORT boolean OCI_API OCI_ElemSetColl2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Coll *value, ExceptionSink* xsink);
-OCI_EXPORT boolean OCI_API OCI_CollFree2(OCI_Library *pOCILib, OCI_Coll *coll);
-OCI_EXPORT OCI_Elem * OCI_API OCI_CollGetAt2(OCI_Library *pOCILib, OCI_Coll *coll, unsigned int index);
-OCI_EXPORT boolean OCI_API OCI_CollAppend2(OCI_Library *pOCILib, OCI_Coll *coll, OCI_Elem *elem);
-OCI_EXPORT OCI_Timestamp * OCI_API OCI_TimestampCreate2(OCI_Library *pOCILib, OCI_Connection *con, unsigned int type);
-OCI_EXPORT boolean OCI_API OCI_ElemSetTimestamp2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Timestamp *value);
-OCI_EXPORT boolean OCI_API OCI_TimestampAssign2(OCI_Library *pOCILib, OCI_Timestamp *tmsp, OCI_Timestamp *tmsp_src);
-OCI_EXPORT boolean OCI_API OCI_TimestampConstruct2(OCI_Library *pOCILib, OCI_Timestamp *tmsp, int year,int month, int day, int hour,  int min, int sec, int fsec, const mtext *timezone);
+OCI_EXPORT boolean OCI_API OCI_CollFree2(OCI_Library *pOCILib, OCI_Coll *coll, ExceptionSink* xsink);
+OCI_EXPORT OCI_Elem * OCI_API OCI_CollGetAt2(OCI_Library *pOCILib, OCI_Coll *coll, unsigned int index, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_CollAppend2(OCI_Library *pOCILib, OCI_Coll *coll, OCI_Elem *elem, ExceptionSink* xsink);
+OCI_EXPORT OCI_Timestamp * OCI_API OCI_TimestampCreate2(OCI_Library *pOCILib, OCI_Connection *con, unsigned int type, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_ElemSetTimestamp2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Timestamp *value, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_TimestampAssign2(OCI_Library *pOCILib, OCI_Timestamp *tmsp, OCI_Timestamp *tmsp_src, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_TimestampConstruct2(OCI_Library *pOCILib, OCI_Timestamp *tmsp, int year,int month, int day, int hour,  int min, int sec, int fsec, const mtext *timezone, ExceptionSink* xsink);
 OCI_EXPORT OCI_Timestamp * OCI_API  OCI_ElemGetTimestamp2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT OCI_Timestamp * OCI_API OCI_ObjectGetTimestamp2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
-OCI_EXPORT OCI_Interval * OCI_API OCI_IntervalCreate2(OCI_Library *pOCILib, OCI_Connection *con, unsigned int type);
-OCI_EXPORT boolean OCI_API OCI_IntervalAssign2(OCI_Library *pOCILib, OCI_Interval *itv,  OCI_Interval *itv_src);
-OCI_EXPORT boolean OCI_API OCI_IntervalGetDaySecond2(OCI_Library *pOCILib, OCI_Interval *itv, int *day, int *hour, int *min, int *sec, int *fsec);
-OCI_EXPORT boolean OCI_API OCI_IntervalGetYearMonth2(OCI_Library *pOCILib, OCI_Interval *itv, int *year, int *month);
-OCI_EXPORT boolean OCI_API OCI_IntervalSetDaySecond2(OCI_Library *pOCILib, OCI_Interval *itv, int day,int hour, int min, int sec, int fsec);
-OCI_EXPORT boolean OCI_API OCI_IntervalSetYearMonth2(OCI_Library *pOCILib, OCI_Interval *itv, int year, int month);
-OCI_EXPORT OCI_Interval * OCI_API OCI_ObjectGetInterval2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
+OCI_EXPORT OCI_Timestamp * OCI_API OCI_ObjectGetTimestamp2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
+OCI_EXPORT OCI_Interval * OCI_API OCI_IntervalCreate2(OCI_Library *pOCILib, OCI_Connection *con, unsigned int type, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_IntervalAssign2(OCI_Library *pOCILib, OCI_Interval *itv,  OCI_Interval *itv_src, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_IntervalGetDaySecond2(OCI_Library *pOCILib, OCI_Interval *itv, int *day, int *hour, int *min, int *sec, int *fsec, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_IntervalGetYearMonth2(OCI_Library *pOCILib, OCI_Interval *itv, int *year, int *month, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_IntervalSetDaySecond2(OCI_Library *pOCILib, OCI_Interval *itv, int day,int hour, int min, int sec, int fsec, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_IntervalSetYearMonth2(OCI_Library *pOCILib, OCI_Interval *itv, int year, int month, ExceptionSink* xsink);
+OCI_EXPORT OCI_Interval * OCI_API OCI_ObjectGetInterval2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
 OCI_EXPORT OCI_Interval * OCI_API OCI_ElemGetInterval2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT boolean OCI_API OCI_ElemSetInterval2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Interval *value);
-OCI_EXPORT unsigned int OCI_API OCI_CollGetSize2(OCI_Library *pOCILib, OCI_Coll *coll);
-OCI_EXPORT boolean OCI_API OCI_LobIsTemporary2(OCI_Library *pOCILib, OCI_Lob *lob);
-OCI_EXPORT boolean OCI_ColumnDescribe2(OCI_Library *pOCILib, OCI_Column *col, OCI_Connection *con, OCI_Statement *stmt, void *handle, int index, int ptype);
+OCI_EXPORT boolean OCI_API OCI_ElemSetInterval2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Interval *value, ExceptionSink* xsink);
+OCI_EXPORT unsigned int OCI_API OCI_CollGetSize2(OCI_Library *pOCILib, OCI_Coll *coll, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_API OCI_LobIsTemporary2(OCI_Library *pOCILib, OCI_Lob *lob, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_ColumnDescribe2(OCI_Library *pOCILib, OCI_Column *col, OCI_Connection *con, OCI_Statement *stmt, void *handle, int index, int ptype, ExceptionSink* xsink);
 OCI_EXPORT boolean OCI_ColumnMap2(OCI_Library *pOCILib, OCI_Column *col, OCI_Statement *stmt);
 OCI_EXPORT boolean OCI_KeyMapFree2(OCI_Library *pOCILib);
-OCI_EXPORT void OCI_ExceptionRaise2(OCI_Library *pOCILib, OCI_Error *err, ExceptionSink* xsink = 0);
-OCI_EXPORT void OCI_ExceptionOCI2(OCI_Library *pOCILib, OCIError *p_err, OCI_Connection *con, OCI_Statement *stmt, boolean warning, ExceptionSink* xsink = 0);
-OCI_EXPORT void OCI_ExceptionNotInitialized2(OCI_Library *pOCILib);
-OCI_EXPORT void OCI_ExceptionLoadingSharedLib2(OCI_Library *pOCILib);
-OCI_EXPORT void OCI_ExceptionLoadingSymbols2(OCI_Library *pOCILib);
-OCI_EXPORT void OCI_ExceptionNotMultithreaded2(OCI_Library *pOCILib);
-OCI_EXPORT void OCI_ExceptionNullPointer2(OCI_Library *pOCILib, int type);
-OCI_EXPORT void OCI_ExceptionMemory2(OCI_Library *pOCILib, int type, size_t nb_bytes, OCI_Connection *con, OCI_Statement *stmt);
-OCI_EXPORT void OCI_ExceptionNotAvailable2(OCI_Library *pOCILib, OCI_Connection *con, int feature);
-OCI_EXPORT void OCI_ExceptionDatatypeNotSupported2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Statement *stmt, int code);
-OCI_EXPORT void OCI_ExceptionParsingToken2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Statement *stmt, mtext token);
-OCI_EXPORT void OCI_ExceptionMappingArgument2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Statement *stmt, int arg);
-OCI_EXPORT void OCI_ExceptionOutOfBounds2(OCI_Library *pOCILib, OCI_Connection *con, int value);
-OCI_EXPORT void OCI_ExceptionUnfreedData2(OCI_Library *pOCILib, int type_elem, int nb_elem);
-OCI_EXPORT void OCI_ExceptionMaxBind2(OCI_Library *pOCILib, OCI_Statement *stmt);
+OCI_EXPORT void OCI_ExceptionRaise2(OCI_Library *pOCILib, OCI_Error *err, ExceptionSink* xsink);
+OCI_EXPORT void OCI_ExceptionOCI2(OCI_Library *pOCILib, OCIError *p_err, OCI_Connection *con, OCI_Statement *stmt, boolean warning, ExceptionSink* xsink);
+OCI_EXPORT void OCI_ExceptionNotInitialized2(OCI_Library *pOCILib, ExceptionSink* xsink);
+//OCI_EXPORT void OCI_ExceptionLoadingSharedLib2(OCI_Library *pOCILib);
+//OCI_EXPORT void OCI_ExceptionLoadingSymbols2(OCI_Library *pOCILib);
+//OCI_EXPORT void OCI_ExceptionNotMultithreaded2(OCI_Library *pOCILib);
+OCI_EXPORT void OCI_ExceptionNullPointer2(OCI_Library *pOCILib, int type, ExceptionSink* xsink);
+//OCI_EXPORT void OCI_ExceptionMemory2(OCI_Library *pOCILib, int type, size_t nb_bytes, OCI_Connection *con, OCI_Statement *stmt);
+OCI_EXPORT void OCI_ExceptionNotAvailable2(OCI_Library *pOCILib, OCI_Connection *con, int feature, ExceptionSink* xsink);
+//OCI_EXPORT void OCI_ExceptionDatatypeNotSupported2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Statement *stmt, int code);
+//OCI_EXPORT void OCI_ExceptionParsingToken2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Statement *stmt, mtext token);
+//OCI_EXPORT void OCI_ExceptionMappingArgument2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Statement *stmt, int arg);
+OCI_EXPORT void OCI_ExceptionOutOfBounds2(OCI_Library *pOCILib, OCI_Connection *con, int value, ExceptionSink* xsink);
+OCI_EXPORT void OCI_ExceptionUnfreedData2(OCI_Library *pOCILib, int type_elem, int nb_elem, ExceptionSink* xsink);
+//OCI_EXPORT void OCI_ExceptionMaxBind2(OCI_Library *pOCILib, OCI_Statement *stmt);
 OCI_EXPORT void OCI_ExceptionAttributeNotFound2(OCI_Library *pOCILib, OCI_Connection *con, const mtext *attr, ExceptionSink* xsink);
-OCI_EXPORT void OCI_ExceptionMinimumValue2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Statement *stmt, int min);
-OCI_EXPORT void OCI_ExceptionTypeNotCompatible2(OCI_Library *pOCILib, OCI_Connection *con);
-OCI_EXPORT void OCI_ExceptionStatementState2(OCI_Library *pOCILib, OCI_Statement *stmt, int state);
-OCI_EXPORT void OCI_ExceptionStatementNotScrollable2(OCI_Library *pOCILib, OCI_Statement *stmt);
-OCI_EXPORT void OCI_ExceptionBindAlreadyUsed2(OCI_Library *pOCILib, OCI_Statement *stmt, const mtext * bind);
-OCI_EXPORT void OCI_ExceptionBindArraySize2(OCI_Library *pOCILib, OCI_Statement *stmt, unsigned int maxsize, unsigned int cursize, unsigned int newsize);
-OCI_EXPORT void OCI_ExceptionDirPathColNotFound2(OCI_Library *pOCILib, OCI_DirPath *dp, const mtext * column, const mtext *table);
-OCI_EXPORT void OCI_ExceptionDirPathState2(OCI_Library *pOCILib, OCI_DirPath *dp, int state);
-OCI_EXPORT void OCI_ExceptionOCIEnvironment2(OCI_Library *pOCILib);
-OCI_EXPORT boolean OCI_API OCI_DateFromCTime2(OCI_Library *pOCILib, OCI_Date *date, struct tm *ptm, time_t t);
+OCI_EXPORT void OCI_ExceptionMinimumValue2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Statement *stmt, int min, ExceptionSink* xsink);
+OCI_EXPORT void OCI_ExceptionTypeNotCompatible2(OCI_Library *pOCILib, OCI_Connection *con, ExceptionSink* xsink);
+//OCI_EXPORT void OCI_ExceptionStatementState2(OCI_Library *pOCILib, OCI_Statement *stmt, int state);
+//OCI_EXPORT void OCI_ExceptionStatementNotScrollable2(OCI_Library *pOCILib, OCI_Statement *stmt);
+//OCI_EXPORT void OCI_ExceptionBindAlreadyUsed2(OCI_Library *pOCILib, OCI_Statement *stmt, const mtext * bind);
+//OCI_EXPORT void OCI_ExceptionBindArraySize2(OCI_Library *pOCILib, OCI_Statement *stmt, unsigned int maxsize, unsigned int cursize, unsigned int newsize);
+//OCI_EXPORT void OCI_ExceptionDirPathColNotFound2(OCI_Library *pOCILib, OCI_DirPath *dp, const mtext * column, const mtext *table);
+//OCI_EXPORT void OCI_ExceptionDirPathState2(OCI_Library *pOCILib, OCI_DirPath *dp, int state);
+//OCI_EXPORT void OCI_ExceptionOCIEnvironment2(OCI_Library *pOCILib);
+//OCI_EXPORT boolean OCI_API OCI_DateFromCTime2(OCI_Library *pOCILib, OCI_Date *date, struct tm *ptm, time_t t);
 OCI_EXPORT boolean OCI_API OCI_TimestampFree2(OCI_Library *pOCILib, OCI_Timestamp *tmsp);
-OCI_EXPORT boolean OCI_ElemSetNumber2(OCI_Library *pOCILib, OCI_Elem  *elem, void *value, uword size, uword flag);
+OCI_EXPORT boolean OCI_ElemSetNumberFromString(OCI_Library *pOCILib, OCI_Elem* elem, const char* str, int size, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_ElemSetNumber2(OCI_Library *pOCILib, OCI_Elem  *elem, void *value, uword size, uword flag, ExceptionSink* xsink);
 OCI_EXPORT OCI_Date * OCI_API  OCI_ElemGetDate2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT boolean OCI_API OCI_ElemSetDate2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Date *value);
-OCI_EXPORT boolean OCI_API OCI_ElemSetFile2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_File *value);
-OCI_EXPORT boolean OCI_API OCI_ElemSetRef2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Ref *value);
+OCI_EXPORT boolean OCI_API OCI_ElemSetDate2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Date *value, ExceptionSink* xsink);
+//OCI_EXPORT boolean OCI_API OCI_ElemSetFile2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_File *value);
+//OCI_EXPORT boolean OCI_API OCI_ElemSetRef2(OCI_Library *pOCILib, OCI_Elem *elem, OCI_Ref *value);
 OCI_EXPORT boolean OCI_API OCI_ElemIsNull2(OCI_Library *pOCILib, OCI_Elem *elem);
 OCI_EXPORT boolean OCI_API OCI_ElemSetNull2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT boolean OCI_ElemGetNumber2(OCI_Library *pOCILib, OCI_Elem *elem, void *value, uword size, uword flag);
-OCI_EXPORT OCI_Elem * OCI_API OCI_ElemCreate2(OCI_Library *pOCILib, OCI_TypeInfo *typinf);
-OCI_EXPORT short OCI_API OCI_ElemGetShort2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT unsigned short OCI_API OCI_ElemGetUnsignedShort2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT int OCI_API OCI_ElemGetInt2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT unsigned int OCI_API OCI_ElemGetUnsignedInt2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT int64 OCI_API OCI_ElemGetBigInt2(OCI_Library *pOCILib, OCI_Elem *elem);
-OCI_EXPORT big_uint OCI_API OCI_ElemGetUnsignedBigInt2(OCI_Library *pOCILib, OCI_Elem *elem);
+OCI_EXPORT boolean OCI_ElemGetNumber2(OCI_Library *pOCILib, OCI_Elem *elem, void *value, uword size, uword flag, ExceptionSink* xsink);
+OCI_EXPORT OCI_Elem * OCI_API OCI_ElemCreate2(OCI_Library *pOCILib, OCI_TypeInfo *typinf, ExceptionSink* xsink);
+//OCI_EXPORT short OCI_API OCI_ElemGetShort2(OCI_Library *pOCILib, OCI_Elem *elem);
+//OCI_EXPORT unsigned short OCI_API OCI_ElemGetUnsignedShort2(OCI_Library *pOCILib, OCI_Elem *elem);
+//OCI_EXPORT int OCI_API OCI_ElemGetInt2(OCI_Library *pOCILib, OCI_Elem *elem);
+//OCI_EXPORT unsigned int OCI_API OCI_ElemGetUnsignedInt2(OCI_Library *pOCILib, OCI_Elem *elem);
+OCI_EXPORT int64 OCI_API OCI_ElemGetBigInt2(OCI_Library *pOCILib, OCI_Elem *elem, ExceptionSink* xsink);
+//OCI_EXPORT big_uint OCI_API OCI_ElemGetUnsignedBigInt2(OCI_Library *pOCILib, OCI_Elem *elem);
 OCI_EXPORT boolean OCI_API OCI_DatabaseShutdown2(OCI_Library *pOCILib, const mtext *db, const mtext *user, const mtext *pwd, unsigned int sess_mode, unsigned int shut_mode, unsigned int shut_flag);
-OCI_EXPORT boolean OCI_NumberGet2(OCI_Library *pOCILib, OCI_Connection *con,  OCINumber *data, void *value, uword size, uword flag);
-OCI_EXPORT boolean OCI_NumberSet2(OCI_Library *pOCILib, OCI_Connection *con,  OCINumber *data, void *value, uword size, uword flag);
-OCI_EXPORT boolean OCI_NumberConvertStr2(OCI_Library *pOCILib, OCI_Connection *con,  OCINumber *num, const dtext *str, int str_size, const mtext* fmt, ub4 fmt_size);
-OCI_EXPORT boolean OCI_NumberGetFromStr2(OCI_Library *pOCILib, OCI_Connection *con,  void *value, uword size, uword type, const dtext *str, int str_size, const mtext* fmt, ub4 fmt_size);
+OCI_EXPORT boolean OCI_NumberGet2(OCI_Library *pOCILib, OCI_Connection *con,  OCINumber *data, void *value, uword size, uword flag, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_NumberSet2(OCI_Library *pOCILib, OCI_Connection *con,  OCINumber *data, void *value, uword size, uword flag, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_NumberConvertStr2(OCI_Library *pOCILib, OCI_Connection *con,  OCINumber *num, const dtext *str, int str_size, const mtext* fmt, ub4 fmt_size, ExceptionSink* xsink);
+OCI_EXPORT boolean OCI_NumberGetFromStr2(OCI_Library *pOCILib, OCI_Connection *con,  void *value, uword size, uword type, const dtext *str, int str_size, const mtext* fmt, ub4 fmt_size, ExceptionSink* xsink);
 OCI_EXPORT boolean OCI_API OCI_TypeInfoFree2(OCI_Library *pOCILib, OCI_TypeInfo *typinf);
 OCI_EXPORT unsigned int OCI_API OCI_TypeInfoGetType2(OCI_Library *pOCILib, OCI_TypeInfo *typinf);
 OCI_EXPORT unsigned int OCI_API OCI_TypeInfoGetColumnCount2(OCI_Library *pOCILib, OCI_TypeInfo *typinf);
-OCI_EXPORT OCI_Column * OCI_API OCI_TypeInfoGetColumn2(OCI_Library *pOCILib, OCI_TypeInfo *typinf, unsigned int index);
+OCI_EXPORT OCI_Column * OCI_API OCI_TypeInfoGetColumn2(OCI_Library *pOCILib, OCI_TypeInfo *typinf, unsigned int index, ExceptionSink* xsink);
 OCI_EXPORT const mtext * OCI_API OCI_TypeInfoGetName2(OCI_Library *pOCILib, OCI_TypeInfo *typinf);
-OCI_EXPORT const mtext * OCI_API OCI_ColumnGetName2(OCI_Library *pOCILib, OCI_Column *col);
+OCI_EXPORT const mtext * OCI_API OCI_ColumnGetName2(OCI_Library *pOCILib, OCI_Column *col, ExceptionSink* xsink);
 OCI_EXPORT unsigned int OCI_API OCI_ColumnGetType2(OCI_Library *pOCILib, OCI_Column *col);
 OCI_EXPORT unsigned int OCI_API OCI_ColumnGetCharsetForm2(OCI_Library *pOCILib, OCI_Column *col);
 OCI_EXPORT unsigned int OCI_API OCI_ColumnGetSize2(OCI_Library *pOCILib, OCI_Column *col);
@@ -1016,7 +1023,7 @@ OCI_EXPORT unsigned int OCI_API OCI_ColumnGetFullSQLType2(OCI_Library *pOCILib,
 OCI_EXPORT OCI_TypeInfo * OCI_API OCI_ColumnGetTypeInfo2(OCI_Library *pOCILib, OCI_Column *col);
 OCI_EXPORT unsigned int OCI_API OCI_ColumnGetSubType2(OCI_Library *pOCILib, OCI_Column *col);
 OCI_EXPORT OCI_Error * OCI_ErrorGet2(OCI_Library *pOCILib, boolean check, boolean warning);
-OCI_EXPORT int OCI_API OCI_ErrorGetInternalCode2(OCI_Library *pOCILib, OCI_Error *err);
+OCI_EXPORT int OCI_API OCI_ErrorGetInternalCode2(OCI_Error *err);
 OCI_EXPORT OCI_List * OCI_ListCreate2(OCI_Library *pOCILib, int type);
 void * OCI_MemAlloc2(OCI_Library *pOCILib, int ptr_type, size_t block_size, size_t block_count, boolean zero_fill);
 void * OCI_MemRealloc2(OCI_Library *pOCILib, void * ptr_mem, int ptr_type, size_t block_size, size_t block_count);
@@ -1028,86 +1035,87 @@ sword OCI_DescriptorFree2(OCI_Library *pOCILib, dvoid *descp, CONST ub4 type);
 sword OCI_DescriptorArrayFree2(OCI_Library *pOCILib, dvoid **descp, CONST ub4 type, ub4 nb_elem);
 sword OCI_ObjectNew2(OCI_Library *pOCILib, OCIEnv *env, OCIError *err, CONST OCISvcCtx *svc, OCITypeCode typecode, OCIType *tdo, dvoid *table, OCIDuration duration, boolean value, dvoid **instance);
 sword OCI_OCIObjectFree2(OCI_Library *pOCILib, OCIEnv *env, OCIError *err, dvoid *instance, ub2 flags);
-int OCI_ObjectGetAttrIndex2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, int type, ExceptionSink* xsink = 0);
-boolean OCI_ObjectSetNumber2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, void *value, uword size, uword flag);
-boolean OCI_ObjectGetNumber2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, void *value, uword size, uword flag, ExceptionSink* xsink = 0);
-OCI_Date * OCI_API OCI_ObjectGetDate2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
+int OCI_ObjectGetAttrIndex2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, int type, ExceptionSink* xsink);
+boolean OCI_ObjectSetNumberFromString(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, const char* str, int size, ExceptionSink* xsink);
+boolean OCI_ObjectSetNumber2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, void *value, uword size, uword flag, ExceptionSink* xsink);
+boolean OCI_ObjectGetNumber2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, void *value, uword size, uword flag, ExceptionSink* xsink);
+OCI_Date * OCI_API OCI_ObjectGetDate2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
 boolean OCI_API OCI_ObjectSetLob2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Lob *value, ExceptionSink* xsink);
-boolean OCI_API OCI_ObjectSetFile2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_File *value);
-boolean OCI_API OCI_ObjectSetNull2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
-boolean OCI_API OCI_ObjectIsNull2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
+//boolean OCI_API OCI_ObjectSetFile2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_File *value);
+boolean OCI_API OCI_ObjectSetNull2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
+boolean OCI_API OCI_ObjectIsNull2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
 OCI_TypeInfo * OCI_API OCI_ObjectGetTypeInfo2(OCI_Library *pOCILib, OCI_Object *obj);
 unsigned int OCI_API OCI_ObjectGetType2(OCI_Library *pOCILib, OCI_Object *obj);
 boolean OCI_API OCI_ObjectGetStruct2(OCI_Library *pOCILib, OCI_Object *obj, void **pp_struct, void** pp_ind);
-OCI_Ref * OCI_API OCI_ObjectGetRef2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
-boolean OCI_API OCI_ObjectSetDate2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Date *value);
+//OCI_Ref * OCI_API OCI_ObjectGetRef2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
+boolean OCI_API OCI_ObjectSetDate2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, OCI_Date *value, ExceptionSink* xsink);
 OCI_Date * OCI_DateInit2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Date **pdate, OCIDate *buffer, boolean allocate, boolean ansi);
-OCI_Elem * OCI_ElemInit2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Elem **pelem, void *handle, OCIInd *pind, OCI_TypeInfo *typinf);
+OCI_Elem * OCI_ElemInit2(OCI_Library *pOCILib, OCI_Connection *con, OCI_Elem **pelem, void *handle, OCIInd *pind, OCI_TypeInfo *typinf, ExceptionSink* xsink);
 char * ocistrdup2(OCI_Library *pOCILib, const char * src);
 wchar_t * ociwcsdup2(OCI_Library *pOCILib, const wchar_t * src);
-boolean OCI_API OCI_ObjectSetDouble2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, double value);
-boolean OCI_API OCI_ObjectSetUnsignedBigInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, big_uint value);
-boolean OCI_API OCI_ObjectSetBigInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, big_int value);
-boolean OCI_API OCI_ObjectSetUnsignedInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, unsigned int value);
-boolean OCI_API OCI_ObjectSetInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, int value);
-boolean OCI_API OCI_ObjectSetUnsignedShort2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, unsigned short value);
-boolean OCI_API OCI_ObjectSetShort2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, short value);
-boolean OCI_API OCI_LobFree(OCI_Library *pOCILib, OCI_Lob *lob);
+boolean OCI_API OCI_ObjectSetDouble2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, double value, ExceptionSink* xsink);
+//boolean OCI_API OCI_ObjectSetUnsignedBigInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, big_uint value);
+boolean OCI_API OCI_ObjectSetBigInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, big_int value, ExceptionSink* xsink);
+//boolean OCI_API OCI_ObjectSetUnsignedInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, unsigned int value);
+//boolean OCI_API OCI_ObjectSetInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, int value);
+//boolean OCI_API OCI_ObjectSetUnsignedShort2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, unsigned short value);
+//boolean OCI_API OCI_ObjectSetShort2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, short value);
+boolean OCI_API OCI_LobFree(OCI_Library *pOCILib, OCI_Lob *lob, ExceptionSink* xsink);
 unsigned int OCI_API OCI_LobGetType(OCI_Library *pOCILib, OCI_Lob *lob);
-boolean OCI_API OCI_LobSeek(OCI_Library *pOCILib, OCI_Lob *lob, big_uint offset, unsigned int mode);
-big_uint OCI_API OCI_LobGetOffset(OCI_Library *pOCILib, OCI_Lob *lob);
+//boolean OCI_API OCI_LobSeek(OCI_Library *pOCILib, OCI_Lob *lob, big_uint offset, unsigned int mode);
+//big_uint OCI_API OCI_LobGetOffset(OCI_Library *pOCILib, OCI_Lob *lob);
 boolean OCI_API OCI_LobRead2(OCI_Library *pOCILib, OCI_Lob *lob, void *buffer, unsigned int *char_count, unsigned int *byte_count, ExceptionSink* xsink);
 boolean OCI_API OCI_LobWrite2(OCI_Library *pOCILib, OCI_Lob *lob, void *buffer, unsigned int *char_count, unsigned int *byte_count, ExceptionSink* xsink);
-boolean OCI_API OCI_LobTruncate(OCI_Library *pOCILib, OCI_Lob *lob, big_uint size);
-big_uint OCI_API OCI_LobErase(OCI_Library *pOCILib, OCI_Lob *lob, big_uint offset, big_uint size);
-big_uint OCI_API OCI_LobGetLength(OCI_Library *pOCILib, OCI_Lob *lob);
-unsigned int OCI_API OCI_LobGetChunkSize(OCI_Library *pOCILib, OCI_Lob *lob);
-boolean OCI_API OCI_LobCopy(OCI_Library *pOCILib, OCI_Lob *lob, OCI_Lob *lob_src, big_uint offset_dst, big_uint offset_src, big_uint count);
-boolean OCI_API OCI_LobCopyFromFile(OCI_Library *pOCILib, OCI_Lob *lob, OCI_File *file, big_uint offset_dst, big_uint offset_src, big_uint count);
-boolean OCI_API OCI_LobAppend2(OCI_Library *pOCILib, OCI_Lob *lob, void *buffer, unsigned int *char_count, unsigned int *byte_count, ExceptionSink* xsink);
-boolean OCI_API OCI_LobAppendLob(OCI_Library *pOCILib, OCI_Lob *lob, OCI_Lob *lob_src);
+//boolean OCI_API OCI_LobTruncate(OCI_Library *pOCILib, OCI_Lob *lob, big_uint size, ExceptionSink* xsink);
+//big_uint OCI_API OCI_LobErase(OCI_Library *pOCILib, OCI_Lob *lob, big_uint offset, big_uint size);
+big_uint OCI_API OCI_LobGetLength(OCI_Library *pOCILib, OCI_Lob *lob, ExceptionSink* xsink);
+//unsigned int OCI_API OCI_LobGetChunkSize(OCI_Library *pOCILib, OCI_Lob *lob);
+//boolean OCI_API OCI_LobCopy(OCI_Library *pOCILib, OCI_Lob *lob, OCI_Lob *lob_src, big_uint offset_dst, big_uint offset_src, big_uint count);
+//boolean OCI_API OCI_LobCopyFromFile(OCI_Library *pOCILib, OCI_Lob *lob, OCI_File *file, big_uint offset_dst, big_uint offset_src, big_uint count);
+//boolean OCI_API OCI_LobAppend2(OCI_Library *pOCILib, OCI_Lob *lob, void *buffer, unsigned int *char_count, unsigned int *byte_count, ExceptionSink* xsink);
+//boolean OCI_API OCI_LobAppendLob(OCI_Library *pOCILib, OCI_Lob *lob, OCI_Lob *lob_src);
 boolean OCI_API OCI_LobOpen(OCI_Library *pOCILib, OCI_Lob *lob, unsigned int mode);
 boolean OCI_API OCI_LobClose(OCI_Library *pOCILib, OCI_Lob *lob);
-big_uint OCI_API OCI_LobGetMaxSize(OCI_Library *pOCILib, OCI_Lob *lob);
-boolean OCI_API OCI_LobFlush(OCI_Library *pOCILib, OCI_Lob *lob);
-boolean OCI_API OCI_LobEnableBuffering(OCI_Library *pOCILib, OCI_Lob *lob, boolean value);
+//big_uint OCI_API OCI_LobGetMaxSize(OCI_Library *pOCILib, OCI_Lob *lob);
+//boolean OCI_API OCI_LobFlush(OCI_Library *pOCILib, OCI_Lob *lob);
+//boolean OCI_API OCI_LobEnableBuffering(OCI_Library *pOCILib, OCI_Lob *lob, boolean value);
 OCI_Date * OCI_API OCI_DateCreate(OCI_Library *pOCILib, OCI_Connection *con);
-boolean OCI_API OCI_DateAddDays(OCI_Library *pOCILib, OCI_Date *date, int nb);
-boolean OCI_API OCI_DateAddMonths(OCI_Library *pOCILib, OCI_Date *date, int nb);
-boolean OCI_API OCI_DateAssign(OCI_Library *pOCILib, OCI_Date *date, OCI_Date *date_src);
-int OCI_API OCI_DateCheck(OCI_Library *pOCILib, OCI_Date *date);
-int OCI_API OCI_DateCompare(OCI_Library *pOCILib, OCI_Date *date, OCI_Date *date2);
-int OCI_API OCI_DateDaysBetween(OCI_Library *pOCILib, OCI_Date *date, OCI_Date *date2);
-boolean OCI_API OCI_DateFromText(OCI_Library *pOCILib, OCI_Date *date, const mtext *str, const mtext *fmt);
-boolean OCI_API OCI_DateGetDate(OCI_Library *pOCILib, OCI_Date *date, int *year, int *month, int *day);
-boolean OCI_API OCI_DateGetTime(OCI_Library *pOCILib, OCI_Date *date, int *hour, int *min, int *sec);
-boolean OCI_API OCI_DateGetDateTime(OCI_Library *pOCILib, OCI_Date *date, int *year, int *month, int *day, int *hour, int *min, int *sec);
-boolean OCI_API OCI_DateLastDay(OCI_Library *pOCILib, OCI_Date *date);
-boolean OCI_API OCI_DateNextDay(OCI_Library *pOCILib, OCI_Date *date, const mtext *day);
+//boolean OCI_API OCI_DateAddDays(OCI_Library *pOCILib, OCI_Date *date, int nb);
+//boolean OCI_API OCI_DateAddMonths(OCI_Library *pOCILib, OCI_Date *date, int nb);
+boolean OCI_API OCI_DateAssign(OCI_Library *pOCILib, OCI_Date *date, OCI_Date *date_src, ExceptionSink* xsink);
+//int OCI_API OCI_DateCheck(OCI_Library *pOCILib, OCI_Date *date);
+//int OCI_API OCI_DateCompare(OCI_Library *pOCILib, OCI_Date *date, OCI_Date *date2);
+//int OCI_API OCI_DateDaysBetween(OCI_Library *pOCILib, OCI_Date *date, OCI_Date *date2);
+//boolean OCI_API OCI_DateFromText(OCI_Library *pOCILib, OCI_Date *date, const mtext *str, const mtext *fmt);
+//boolean OCI_API OCI_DateGetDate(OCI_Library *pOCILib, OCI_Date *date, int *year, int *month, int *day);
+//boolean OCI_API OCI_DateGetTime(OCI_Library *pOCILib, OCI_Date *date, int *hour, int *min, int *sec);
+//boolean OCI_API OCI_DateGetDateTime(OCI_Library *pOCILib, OCI_Date *date, int *year, int *month, int *day, int *hour, int *min, int *sec);
+//boolean OCI_API OCI_DateLastDay(OCI_Library *pOCILib, OCI_Date *date);
+//boolean OCI_API OCI_DateNextDay(OCI_Library *pOCILib, OCI_Date *date, const mtext *day);
 boolean OCI_API OCI_DateSetDate(OCI_Library *pOCILib, OCI_Date *date, int year, int month, int day);
 boolean OCI_API OCI_DateSetTime(OCI_Library *pOCILib, OCI_Date *date, int hour, int min, int sec);
 boolean OCI_API OCI_DateSetDateTime(OCI_Library *pOCILib, OCI_Date *date, int year, int month, int day, int hour, int min, int sec);
-boolean OCI_API OCI_DateSysDate(OCI_Library *pOCILib, OCI_Date *date);
-boolean OCI_API OCI_DateToText(OCI_Library *pOCILib, OCI_Date *date, const mtext *fmt, int size, mtext *str);
-boolean OCI_API OCI_DateZoneToZone(OCI_Library *pOCILib, OCI_Date *date, const mtext *zone1, const mtext *zone2);
-boolean OCI_API OCI_DateToCTime(OCI_Library *pOCILib, OCI_Date *date, struct tm *ptm, time_t *pt);
+//boolean OCI_API OCI_DateSysDate(OCI_Library *pOCILib, OCI_Date *date);
+//boolean OCI_API OCI_DateToText(OCI_Library *pOCILib, OCI_Date *date, const mtext *fmt, int size, mtext *str);
+//boolean OCI_API OCI_DateZoneToZone(OCI_Library *pOCILib, OCI_Date *date, const mtext *zone1, const mtext *zone2);
+//boolean OCI_API OCI_DateToCTime(OCI_Library *pOCILib, OCI_Date *date, struct tm *ptm, time_t *pt);
 boolean OCI_API OCI_DateFree(OCI_Library *pOCILib, OCI_Date *date);
-boolean OCI_API OCI_ElemSetShort(OCI_Library *pOCILib, OCI_Elem *elem, short value);
-boolean OCI_API OCI_ElemSetUnsignedShort(OCI_Library *pOCILib, OCI_Elem *elem, unsigned short value);
-boolean OCI_API OCI_ElemSetInt(OCI_Library *pOCILib, OCI_Elem *elem, int value);
-boolean OCI_API OCI_ElemSetUnsignedInt(OCI_Library *pOCILib, OCI_Elem *elem, unsigned int value);
-boolean OCI_API OCI_ElemSetBigInt(OCI_Library *pOCILib, OCI_Elem *elem, big_int value);
-boolean OCI_API OCI_ElemSetUnsignedBigInt(OCI_Library *pOCILib, OCI_Elem *elem, big_uint value);
-boolean OCI_API OCI_ElemSetDouble(OCI_Library *pOCILib, OCI_Elem *elem, double value);
-boolean OCI_API OCI_ElemFree2(OCI_Library *pOCILib, OCI_Elem *elem);
-double OCI_API OCI_ElemGetDouble2(OCI_Library *pOCILib, OCI_Elem *elem);
-short OCI_API OCI_ObjectGetShort2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
-unsigned short OCI_API OCI_ObjectGetUnsignedShort2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
-int OCI_API OCI_ObjectGetInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
-unsigned int OCI_API OCI_ObjectGetUnsignedInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
-int64 OCI_API OCI_ObjectGetBigInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
-big_uint OCI_API OCI_ObjectGetUnsignedBigInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
-double OCI_API OCI_ObjectGetDouble2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
+//boolean OCI_API OCI_ElemSetShort(OCI_Library *pOCILib, OCI_Elem *elem, short value);
+//boolean OCI_API OCI_ElemSetUnsignedShort(OCI_Library *pOCILib, OCI_Elem *elem, unsigned short value);
+//boolean OCI_API OCI_ElemSetInt(OCI_Library *pOCILib, OCI_Elem *elem, int value);
+//boolean OCI_API OCI_ElemSetUnsignedInt(OCI_Library *pOCILib, OCI_Elem *elem, unsigned int value);
+boolean OCI_API OCI_ElemSetBigInt(OCI_Library *pOCILib, OCI_Elem *elem, big_int value, ExceptionSink* xsink);
+//boolean OCI_API OCI_ElemSetUnsignedBigInt(OCI_Library *pOCILib, OCI_Elem *elem, big_uint value);
+boolean OCI_API OCI_ElemSetDouble(OCI_Library *pOCILib, OCI_Elem *elem, double value, ExceptionSink* xsink);
+boolean OCI_API OCI_ElemFree2(OCI_Library *pOCILib, OCI_Elem *elem, ExceptionSink* xsink);
+double OCI_API OCI_ElemGetDouble2(OCI_Library *pOCILib, OCI_Elem *elem, ExceptionSink* xsink);
+//short OCI_API OCI_ObjectGetShort2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
+//unsigned short OCI_API OCI_ObjectGetUnsignedShort2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
+//int OCI_API OCI_ObjectGetInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
+//unsigned int OCI_API OCI_ObjectGetUnsignedInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
+int64 OCI_API OCI_ObjectGetBigInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
+//big_uint OCI_API OCI_ObjectGetUnsignedBigInt2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr);
+double OCI_API OCI_ObjectGetDouble2(OCI_Library *pOCILib, OCI_Object *obj, const mtext *attr, ExceptionSink* xsink);
 boolean OCI_API OCI_IntervalFree2(OCI_Library *pOCILib, OCI_Interval *itv);
 
 #ifdef __cplusplus
