@@ -24,10 +24,6 @@
 #include "ocilib/object.cpp"
 #include "ocilib/interval.cpp"
 #include "ocilib/date.cpp"
-#include "ocilib/threadkey.cpp"
-#include "ocilib/mutex.cpp"
-#include "ocilib/ref.cpp"
-#include "ocilib/file.cpp"
 #include "ocilib/connection.cpp"
 #include "ocilib/msg.cpp"
 #include "ocilib/enqueue.cpp"
